-- ----------------------
-- KAPITEL 10 (Oracle 21)
-- ----------------------

-- 10.5.1.1 Datenrepräsentation

-- Netzwerk anlegen:
DECLARE
BEGIN
  SDO_NET.CREATE_SDO_NETWORK('OldenburgNetz', 1, FALSE, FALSE);
END;
/

-- Netzwerktabellen bestimmen:
SELECT table_name FROM USER_TABLES WHERE table_name LIKE 'OLDENBURGNETZ%';

-- Hier die Datei OldenburgNetzNodes.sql ausführen.

-- Hier die Datei OldenburgNetzLinks.sql ausführen.

-- Verbindende Strecken konstruieren: 
UPDATE OldenburgNetz_LINK$
SET geometry = SDO_GEOMETRY(2002, NULL, NULL, SDO_ELEM_INFO_ARRAY(1,2,1),
    SDO_ORDINATE_ARRAY(
       (SELECT n.geometry.sdo_point.x FROM OldenburgNetz_NODE$ n
        WHERE node_id = start_node_id),
       (SELECT n.geometry.sdo_point.y FROM OldenburgNetz_NODE$ n
        WHERE node_id = start_node_id),
       (SELECT n.geometry.sdo_point.x FROM OldenburgNetz_NODE$ n
        WHERE node_id = end_node_id),
       (SELECT n.geometry.sdo_point.y FROM OldenburgNetz_NODE$ n
        WHERE node_id = end_node_id))
    );

UPDATE OldenburgNetz_LINK$
SET cost = link_type * SDO_GEOM.SDO_LENGTH(geometry,0.5);

COMMIT;

-- Metadaten einfügen:
INSERT INTO USER_SDO_GEOM_METADATA VALUES (
  'OLDENBURGNETZ_NODE$','GEOMETRY',
  SDO_DIM_ARRAY(SDO_DIM_ELEMENT('X', 0,24000, 0.5),
                SDO_DIM_ELEMENT('Y', 0,27100, 0.5)),  NULL );
INSERT INTO USER_SDO_GEOM_METADATA VALUES (
  'OLDENBURGNETZ_LINK$', 'GEOMETRY',
  SDO_DIM_ARRAY(SDO_DIM_ELEMENT('X', 0,24000, 0.5),
                SDO_DIM_ELEMENT('Y', 0,27100, 0.5)),  NULL );
INSERT INTO USER_SDO_GEOM_METADATA VALUES (
  'OLDENBURGNETZ_PATH$', 'GEOMETRY',
  SDO_DIM_ARRAY(SDO_DIM_ELEMENT('X', 0,24000, 0.5),
                SDO_DIM_ELEMENT('Y', 0,27100, 0.5)),  NULL );
COMMIT;

-- räumliche Indexe anlegen:
CREATE INDEX olnet_node_idx ON OldenburgNetz_NODE$(geometry)
INDEXTYPE IS MDSYS.SPATIAL_INDEX_V2;

CREATE INDEX olnet_link_idx ON OldenburgNetz_LINK$(geometry)
INDEXTYPE IS MDSYS.SPATIAL_INDEX_V2;

CREATE INDEX olnet_path_idx ON OldenburgNetz_PATH$(geometry)
INDEXTYPE IS MDSYS.SPATIAL_INDEX_V2;

-- Nach der Berechnung des kürzesten Weg über Java:

-- Weg aus Wegtabelle abfragen:
SELECT path_id, cost, p.geometry FROM OldenburgNetz_PATH$ p;

-- Weglänge aus Verknüpfungstabelle berechnen:
SELECT sum(l.cost)
FROM OldenburgNetz_PLINK$ pl INNER JOIN OldenburgNetz_LINK$ l
ON pl.link_id = l.link_id
GROUP BY path_id;

-- 10.5.2.2 Nutzung über SQL

-- Topology anlegen:
CALL SDO_TOPO.CREATE_TOPOLOGY('GeoDbLandTP', 0.001, NULL);

-- Metadaten abrufen:
SELECT topology, topology_id FROM USER_SDO_TOPO_METADATA;

-- Tabellennamen abrufen:
SELECT table_name FROM USER_TABLES WHERE table_name LIKE 'GEODBLANDTP%';

-- Geo-Metadaten abrufen:
SELECT * FROM USER_SDO_GEOM_METADATA WHERE table_name LIKE 'GEODBLANDTP%';

UPDATE USER_SDO_GEOM_METADATA
SET diminfo = SDO_DIM_ARRAY(SDO_DIM_ELEMENT('X',0,20,0.001),
                            SDO_DIM_ELEMENT('Y',0,20,0.001) )
WHERE table_name LIKE 'GEODBLANDTP%';

COMMIT;

CREATE TABLE GeoDbLandNutzung (
  id       INTEGER,                          -- ID
  nutzung  VARCHAR(20),                      -- Nutzungsart
  gebiet   SDO_TOPO_GEOMETRY,                -- topologische Geometrie
  CONSTRAINT pk_gdblandnutz PRIMARY KEY(id)  -- Primärschlüssel
);

-- Tabelle bekannt geben:
CALL SDO_TOPO.ADD_TOPO_GEOMETRY_LAYER(
                    'GeoDbLandTP', 'GeoDbLandNutzung','gebiet', 'POLYGON');
-- Metadaten abrufen:
SELECT topology_id, table_name, column_name, tg_layer_id, tg_layer_type
FROM USER_SDO_TOPO_METADATA WHERE topology = 'GEODBLANDTP';

INSERT INTO GeoDbLandTP_Node$ VALUES (1,1,NULL,SDO_GEOMETRY('POINT(1 1)'));
COMMIT;

-- Masche f-1:
INSERT INTO GeoDbLandTP_Face$
VALUES (-1, NULL, SDO_LIST_TYPE(-1), SDO_LIST_TYPE(), NULL);

-- Masche f1:
INSERT INTO GeoDbLandTP_Face$ VALUES (1, 1, SDO_LIST_TYPE(), SDO_LIST_TYPE(),
          SDO_GEOMETRY(2003, NULL, NULL,
          SDO_ELEM_INFO_ARRAY(1,1003,3), SDO_ORDINATE_ARRAY(1,1,19,20)));

INSERT INTO GeoDbLandTP_Edge$
VALUES (1, 1,1, 1,1,-1,-1, 1,-1,
        SDO_GEOMETRY('LINESTRING(1 1,19 1,19 20,1 20,1 1)'));
COMMIT;

INSERT INTO GeoDbLandNutzung (id,nutzung,gebiet)
VALUES (6,'Weideland',SDO_TOPO_GEOMETRY('GeoDbLandTP',3,1,
                          SDO_TOPO_OBJECT_ARRAY (SDO_TOPO_OBJECT(1,3))) );
COMMIT;

SELECT * FROM GeoDbLandTP_RELATION$;

SELECT nutzung, l.gebiet.get_geometry() FROM GeoDbLandNutzung l;

-- Metadaten aktualisieren:
CALL SDO_TOPO.INITIALIZE_METADATA('GeoDbLandTP');
-- Sequenzgeneratoren überprüfen:
SELECT sequence_name, last_number
FROM USER_SEQUENCES
WHERE sequence_name LIKE 'GEODBLANDTP%S';

-- 10.5.2.3 Nutzung über PL/SQL

-- Überprüfung ohne (Fehler-) Meldung:
DECLARE
  res VARCHAR2(1024);   -- Antwort
BEGIN
  res := SDO_TOPO_MAP.VALIDATE_TOPOLOGY('GeoDbLandTP');
END;
/
-- Überprüfung mit Fehlermeldung:
UPDATE GeoDbLandTP_Face$ SET face_id = 2 WHERE face_id = 1;
DECLARE
  res VARCHAR2(1024);   -- Antwort
BEGIN
  res := SDO_TOPO_MAP.VALIDATE_TOPOLOGY('GeoDbLandTP');
END;
/

ROLLBACK;

-- TopoMap anlegen:
CALL SDO_TOPO_MAP.CREATE_TOPO_MAP('GeoDbLandTP','GeoDbLandTM');

-- TopoMap laden:
DECLARE
  res VARCHAR2(1024);   -- Antwort
BEGIN
  res := SDO_TOPO_MAP.LOAD_TOPO_MAP('GeoDbLandTM',0,0,20,20,'TRUE');
END;
/

-- Knoten und Schlinge einfügen:
DECLARE 
  id NUMBER(38);     -- ID
BEGIN
  id := SDO_TOPO_MAP.ADD_ISOLATED_NODE(NULL,1,SDO_GEOMETRY('POINT(3 10)'));
  id := SDO_TOPO_MAP.ADD_LOOP(NULL,id,SDO_GEOMETRY(
      'LINESTRING(3 10, 4 5, 9 3, 13 5, 16 8, 17 12, 13 15, 7 14, 3 10)'));
END;
/

-- Korrektheit überprüfen:
DECLARE
  res VARCHAR2(1024);   -- Antwort
BEGIN
  res := SDO_TOPO_MAP.VALIDATE_TOPO_MAP('GeoDbLandTM');
END;
/

-- TopoMap in die Datenbank zurückschreiben:
CALL SDO_TOPO_MAP.COMMIT_TOPO_MAP();

-- Maschen überprüfen:
SELECT face_id, boundary_edge_id, mbr_geometry
FROM GeoDbLandTP_FACE$ ORDER BY face_id;

-- Kanten überprüfen:
SELECT edge_id, left_face_id, right_face_id
FROM GeoDbLandTP_EDGE$ ORDER BY edge_id;

SELECT * FROM GeoDbLandTP_RELATION$;

-- Nur noch die Masche f1 zuordnen:
UPDATE GeoDbLandNutzung SET gebiet = SDO_TOPO_GEOMETRY('GeoDbLandTP',3,1,
                              SDO_TOPO_OBJECT_ARRAY (SDO_TOPO_OBJECT(1,3)))
WHERE nutzung = 'Weideland';

-- Masche f3 setzen:
INSERT INTO GeoDbLandNutzung (id,nutzung,gebiet)
VALUES (8,'Wasser',SDO_TOPO_GEOMETRY('GeoDbLandTP',3,1,
                          SDO_TOPO_OBJECT_ARRAY (SDO_TOPO_OBJECT(3,3))) );
COMMIT;

-- TopoMap freigeben:
CALL SDO_TOPO_MAP.CLEAR_TOPO_MAP('GeoDbLandTM');

-- TopMap laden:
DECLARE
  res VARCHAR2(1024);   -- Antwort
BEGIN
  res := SDO_TOPO_MAP.LOAD_TOPO_MAP('GeoDbLandTM',0,0,20,20,'TRUE');
END;
/

DECLARE 
  id4 NUMBER(38);     -- Knoten-ID von n4
  id5 NUMBER(38);     -- Knoten-ID von n5
  id  NUMBER(38);     -- ID neue Kante
BEGIN
  id4 := SDO_TOPO_MAP.ADD_NODE(NULL,1, SDO_GEOMETRY('POINT(19 12)'),
                               1,'TRUE');
  id5 := SDO_TOPO_MAP.ADD_NODE(NULL,3, SDO_GEOMETRY('POINT(17 12)'),
                               5,'FALSE');
  id := SDO_TOPO_MAP.ADD_EDGE(NULL, id4,id5,
                              SDO_GEOMETRY('LINESTRING(19 12, 17 12)'));
END;
/

DECLARE 
  id3 NUMBER(38);     -- Knoten-ID von n3
  id6 NUMBER(38);     -- Knoten-ID von n6
  id  NUMBER(38);     -- ID neue Kante
BEGIN
  id3 := 3;
  id6 := SDO_TOPO_MAP.ADD_NODE(NULL,4, SDO_GEOMETRY('POINT(1 10)'),
                               2,'TRUE');
  id := SDO_TOPO_MAP.ADD_EDGE(NULL, id3,id6,
                              SDO_GEOMETRY('LINESTRING(3 10, 1 10)'));
END;
/

-- TopoMap in die Datenbank zurückschreiben:
CALL SDO_TOPO_MAP.COMMIT_TOPO_MAP();

-- Maschen anzeigen:
SELECT face_id, mbr_geometry
FROM GeoDbLandTP_FACE$ ORDER BY face_id;

-- Dem Weideland nur noch die Masche f1 zuordnen:
UPDATE GeoDbLandNutzung
SET gebiet = SDO_TOPO_GEOMETRY('GeoDbLandTP',3,1,
                           SDO_TOPO_OBJECT_ARRAY (SDO_TOPO_OBJECT(1,3)))
WHERE nutzung = 'Weideland';

-- Für den Wald die Masche f4 setzen:
INSERT INTO GeoDbLandNutzung (id,nutzung,gebiet)
VALUES (9,'Wald',SDO_TOPO_GEOMETRY('GeoDbLandTP',3,1,
                        SDO_TOPO_OBJECT_ARRAY (SDO_TOPO_OBJECT(4,3))) );
COMMIT;

CALL SDO_TOPO_MAP.DROP_TOPO_MAP('GeoDbLandTM');

-- 10.5.2.4 Nutzung über Java

-- Nachdem die Nutzungsgebiete Bebauung und Weideland über Java eingefügt worden sind:

UPDATE GeoDbLandNutzung
SET gebiet = SDO_TOPO_GEOMETRY('GeoDbLandTP',3,1,
                            SDO_TOPO_OBJECT_ARRAY (SDO_TOPO_OBJECT(3,3)))
WHERE nutzung = 'Wasser';
COMMIT;

-- 10.5.2.5 Direkte Erzeugung von topologischen Geometrien

-- ggf. TopoMap anlegen und laden: 
CALL SDO_TOPO_MAP.CREATE_TOPO_MAP('GeoDbLandTP','GeoDbLandTM');
DECLARE
  res VARCHAR2(1024);   -- Antwort
BEGIN
  res := SDO_TOPO_MAP.LOAD_TOPO_MAP('GeoDbLandTM',0,0,20,20,'TRUE');
END;
/

-- topologisches Objekt einfügen: 
INSERT INTO GeoDbLandNutzung (id,nutzung,gebiet)
VALUES (15,'Sumpf',
    SDO_TOPO_MAP.CREATE_FEATURE('GeoDbLandTP','GeoDbLandNutzung','gebiet',
      SDO_GEOMETRY('POLYGON((1 20, 0 20, 0 0, 19 0, 19 1, 1 1, 1 20))')));
COMMIT;

-- TopoMap in die Datenbank zurückschreiben:
CALL SDO_TOPO_MAP.COMMIT_TOPO_MAP();

-- 10.5.2.6 Anfragen

SELECT n.id, n.nutzung
FROM GeoDbLandNutzung n INNER JOIN GeoDbLandNutzung wasser
ON SDO_ANYINTERACT (n.gebiet, wasser.gebiet) = 'TRUE'
WHERE wasser.nutzung = 'Wasser';

SELECT n.id, n.nutzung
FROM GeoDbLandNutzung n INNER JOIN GeoDbLand haus
ON SDO_ANYINTERACT (n.gebiet, haus.geo) = 'TRUE'
WHERE haus.name = 'Haus';

-- Topologische Geometrie als 2.Parameter:
SELECT n.id, n.nutzung
FROM GeoDbLandNutzung n INNER JOIN GeoDbLandNutzung wasser
ON SDO_TOUCH (n.gebiet, wasser.gebiet) = 'TRUE'
WHERE wasser.nutzung = 'Wasser';

SELECT n.id, n.nutzung
FROM GeoDbLandNutzung n INNER JOIN GeoDbLandNutzung wasser
ON SDO_EQUAL (n.gebiet, wasser.gebiet) = 'TRUE'
WHERE wasser.nutzung = 'Wasser';

-- Geometrie als 2. Parameter:
SELECT id, nutzung
FROM GeoDbLandNutzung
WHERE SDO_CONTAINS (gebiet, SDO_GEOMETRY('POINT(3 3)')) = 'TRUE';

SELECT n.id, n.nutzung
FROM GeoDbLandNutzung n INNER JOIN GeoDbLand haus
ON SDO_EQUAL (n.gebiet, haus.geo) = 'TRUE'
WHERE haus.name = 'Haus';

SELECT id, nutzung FROM GeoDbLandNutzung n
WHERE SDO_COVERS (gebiet, SDO_GEOMETRY(2002, NULL,NULL,
                      SDO_ELEM_INFO_ARRAY(1,2,1),
                      SDO_ORDINATE_ARRAY(9,9, 11,9.7))) = 'TRUE';

SELECT n1.id, n1.nutzung, n2.id, n2.nutzung,
       SDO_TOPO.RELATE(n1.gebiet,n2.gebiet,'DETERMINE')
FROM GeoDbLandNutzung n1 INNER JOIN GeoDbLandNutzung n2
ON n1.id < n2.id
WHERE n2.id <> 15
ORDER BY n1.id, n2.id;
