-- ----------------------
-- KAPITEL 12 (Oracle 21)
-- ----------------------

-- 12.4.1.1 3D-Punkte, 3D-Linienzüge und planare Polygone in 3D

-- Anlegen der Tabelle:
CREATE TABLE GeoDbLand3D (
  id     INTEGER,                          -- Identifikationsnummer
  name   VARCHAR(20),                      -- Objektname
  geo    SDO_GEOMETRY,                     -- Objektgeometrie
  CONSTRAINT pk_gdbland3D PRIMARY KEY(id)  -- Primärschlüssel
);

-- Einfügen der Daten:
INSERT INTO GeoDbLand3D (id,name,geo)
VALUES (1,'Flaggenmast', SDO_GEOMETRY(3001,NULL,
          SDO_POINT_TYPE(10,11,5), NULL,NULL));
INSERT INTO GeoDbLand3D (id,name,geo)
VALUES (2,'Steg',SDO_GEOMETRY(3002,NULL,NULL,
          SDO_ELEM_INFO_ARRAY(1,2,1),
          SDO_ORDINATE_ARRAY(4,4,1, 5,7,2.5, 8,8,2)));
INSERT INTO GeoDbLand3D (id,name,geo)
VALUES (3,'Baumreihe',SDO_GEOMETRY(3005,NULL,NULL,
          SDO_ELEM_INFO_ARRAY(1,1,3),
          SDO_ORDINATE_ARRAY(10,19,5, 12,19,10, 14,19,7)));
INSERT INTO GeoDbLand3D (id,name,geo)
VALUES (6,'Land',SDO_GEOMETRY(3007,NULL,NULL,
          SDO_ELEM_INFO_ARRAY(1,1003,1, 16,2003,1, 43,1003,1),
          SDO_ORDINATE_ARRAY(1,1,1, 19,1,1, 19,20,1, 1,20,1, 1,1,1,
            4,5,1, 3,10,1, 7,14,1, 13,15,1, 17,12,1, 16,8,1, 13,5,1, 9,3,1,
            4,5,1, 7,7,2, 12,7,2, 13,11,2, 10,13,2, 7,10,2, 7,7,2)));

-- Metadaten bekannt geben:
INSERT INTO USER_SDO_GEOM_METADATA
VALUES ('GEODBLAND3D','GEO',
        SDO_DIM_ARRAY(SDO_DIM_ELEMENT('X',0,20,0.001),
                        SDO_DIM_ELEMENT('Y',0,20,0.001),
                        SDO_DIM_ELEMENT('Z',0,10,0.001)) ,
        NULL);
COMMIT;

-- Abfrage Dimension:
SELECT DISTINCT (geo).Get_Dims() FROM GeoDbLand3D;

-- minimal umgebender Quader:
SELECT SDO_GEOM.SDO_MBR(geo)
FROM GeoDbLand3D
WHERE name = 'Steg';

-- Länge:
SELECT name, SDO_GEOM.SDO_LENGTH(geo,0.001)
FROM GeoDbLand3D
WHERE name = 'Steg';

-- Abstand: 
SELECT SDO_GEOM.SDO_DISTANCE (geo,
         SDO_GEOMETRY(3001,NULL,SDO_POINT_TYPE(10,10,3),NULL,NULL), 0.001)
FROM GeoDbLand3D WHERE name = 'Flaggenmast';

-- Prüfung der topologischen Schnittbeziehung
SELECT SDO_GEOM.RELATE (g1.geo, 'ANYINTERACT', g2.geo, 0.001)
FROM GeoDbLand3D g1, GeoDbLand3D g2
WHERE g1.name = 'Flaggenmast' AND g2.name = 'Land';

-- Projektion auf 2D: 
SELECT SDO_CS.MAKE_2D(geo) AS geo2D FROM GeoDbLand3D;

-- 2D-Flaggenmast mit Höhe versehen: 
UPDATE GeoDbLand3D
SET geo = (SELECT SDO_CS.MAKE_3D(geo,5) FROM GeoDbLand WHERE id = 1)
WHERE id = 1;

COMMIT;

-- 12.4.1.2 Räumliche Anfragen

CREATE INDEX GeoDbLand3D_geo_ix
ON GeoDbLand3D(geo)
INDEXTYPE IS MDSYS.SPATIAL_INDEX_V2 PARAMETERS ('SDO_INDX_DIMS=3');

-- Anfrage mit 3D-Strecke (Filterschritt):
SELECT name
FROM GeoDbLand3D
WHERE SDO_FILTER (geo, SDO_GEOMETRY(3002,NULL,NULL,
                    SDO_ELEM_INFO_ARRAY(1,2,1),
                    SDO_ORDINATE_ARRAY(5,7,2.5, 15,16,5))) = 'TRUE';

-- Anfrage mit 3D-Strecke (Verfeinerungsschritt):
SELECT name
FROM GeoDbLand3D
WHERE SDO_ANYINTERACT (geo, SDO_GEOMETRY(3002,NULL,NULL,
                         SDO_ELEM_INFO_ARRAY(1,2,1),
                         SDO_ORDINATE_ARRAY(5,7,2.5, 15,16,5))) = 'TRUE';

-- Nächste-Nachbarn-Anfrage: 
SELECT name, SDO_NN_DISTANCE(1)
FROM GeoDbLand3D
WHERE SDO_NN (geo,
  SDO_GEOMETRY(3001,NULL,SDO_POINT_TYPE(10,10,3),NULL,NULL),
  'sdo_num_res=3',1) = 'TRUE'
ORDER BY SDO_NN_DISTANCE(1);

-- 12.4.1.3 3D-Flächen und Körper

-- Dach als 3D-Fläche einfügen: 
INSERT INTO GeoDbLand3D (id,name,geo)
VALUES (51, 'Hausdach', SDO_GEOMETRY(3003, NULL, NULL,
   SDO_ELEM_INFO_ARRAY(1,1006,1, 1,1003,1, 16,2003,1, 31,1003,1),
   SDO_ORDINATE_ARRAY(
      9,9,6.5, 9,9.5,7, 12,9.5,7, 12,9,6.5, 9,9,6.5,
      10,9.15,6.65, 11,9.15,6.65, 11,9.35,6.85, 10,9.35,6.85, 10,9.15,6.65,
      9,10,6.5, 12,10,6.5, 12,9.5,7, 9,9.5,7, 9,10,6.5) ) );

-- Hauskörper als Quader mittels Eckpunktbeschreibung einfügen: 
INSERT INTO GeoDbLand3D (id,name,geo)
VALUES (52, 'Hauskörper', SDO_GEOMETRY(3008,NULL,NULL,
    SDO_ELEM_INFO_ARRAY(1,1007,3), SDO_ORDINATE_ARRAY(9,9,2, 12,10,6.5) ) );
COMMIT;

SELECT SDO_UTIL.EXTRUDE(
  SDO_GEOMETRY('POLYGON((9 9, 12 9, 12 9.5, 12 10, 9 10, 9 9.5, 9 9))'),
  SDO_NUMBER_ARRAY( 6.5, 6.5, 6.5, 6.5, 6.5, 6.5, 6.5 ), 
  SDO_NUMBER_ARRAY( 6.5, 7, 6.5, 6.5, 7, 6.5, 6.5 ), 
  0.001) FROM DUAL;

SELECT SDO_GEOM.SDO_VOLUME (geo,0.001)
FROM GeoDbLand3D
WHERE name = 'Hauskörper';

-- Test mit 3D-Punkt: 
SELECT name, SDO_GEOM.RELATE (geo, 'ANYINTERACT',
  SDO_GEOMETRY(3001,NULL,SDO_POINT_TYPE(9.5,9.25,6.75),NULL,NULL), 0.001)
FROM GeoDbLand3D
WHERE name LIKE 'Haus%';

-- 3D-Rechteckanfrage = Quaderanfrage: 
SELECT name
FROM GeoDbLand3D
WHERE SDO_ANYINTERACT (geo, SDO_GEOMETRY(3008,NULL,NULL,
        SDO_ELEM_INFO_ARRAY(1,1007,3),
        SDO_ORDINATE_ARRAY(9.5,9,6.7, 10,9.25,7.5))) = 'TRUE';

-- 12.4.2.1 Datenstrukturen und Tabellen

-- Metadatentabelle anlegen: 
CREATE TABLE PWMetaDaten (
  name    VARCHAR(50),                  -- Bezeichner
  pwinfo  SDO_PC,                       -- Metadaten
  CONSTRAINT pk_pwmt PRIMARY KEY(name)  -- Primärschlüssel
);

-- Blocktabelle anlegen: 
CREATE TABLE PWBlockDaten AS
  SELECT * FROM MDSYS.SDO_PC_BLK_TABLE;

-- 12.4.2.2 Speichern einer Punktwolke

CREATE TABLE Quellpunkte (
  rid     NUMBER,                      -- ID
  val_d1  NUMBER,                      -- x-Koordinatenwert
  val_d2  NUMBER,                      -- y-Koordinatenwert
  val_d3  NUMBER,                      -- z-Koordinatenwert
  CONSTRAINT pk_qpkt PRIMARY KEY(rid)  -- Primärschlüssel
);

-- Hier QuellpunktePW.sql ausführen.
COMMIT;

DECLARE
  pwinfo  SDO_PC;  -- das Metadaten-Objekt
BEGIN
  -- Metadaten-Objekt erzeugen: 
  pwinfo := SDO_PC_PKG.INIT( 'PWMETADATEN', 'PWINFO', 'PWBLOCKDATEN',
              'blk_capacity=50',
              SDO_GEOMETRY(3003, NULL, NULL, SDO_ELEM_INFO_ARRAY(1,1003,3),
                SDO_ORDINATE_ARRAY(-2559.079, -13712.342, 2025.776, 
                                   -2091.243, -13478.560, 7245.145) ),
              1e-6, 3);
  -- Metadaten abspeichern: 
  INSERT INTO PWMetaDaten VALUES ('Fassade',pwinfo);
  -- Punkte überführen: 
  SDO_PC_PKG.CREATE_PC (pwinfo,'Quellpunkte');
END;
/

-- 12.4.2.3 Anfragen

-- Umwandlung in 3D-MultiPoint-Geometrien: 
SELECT SDO_PC_PKG.TO_GEOMETRY(points,num_points,3)
FROM PWBlockDaten WHERE blk_id = 0;

-- Umwandlung in 2D-MultiPoint-Geometrien: 
SELECT SDO_PC_PKG.TO_GEOMETRY(points,num_points,2)
FROM PWBlockDaten WHERE blk_id = 0;

SELECT SDO_PC_PKG.TO_GEOMETRY(points,num_points,3)
FROM PWBlockDaten
WHERE SDO_FILTER (blk_extent, SDO_GEOMETRY(3008, NULL, NULL,
                   SDO_ELEM_INFO_ARRAY(1,1007,3),
                   SDO_ORDINATE_ARRAY(-2500,-13700,4000,
                                      -2200,-13500,7000) ) ) = 'TRUE';

SELECT SDO_PC_PKG.TO_GEOMETRY(t.points,t.num_points,3)
FROM TABLE (
  SELECT SDO_PC_PKG.CLIP_PC (pwinfo,                     -- Metadaten-Objekt
            SDO_GEOMETRY(3008, NULL, NULL, SDO_ELEM_INFO_ARRAY(1,1007,3),
               SDO_ORDINATE_ARRAY(-2500,-13700,4000, 
                                  -2200,-13500,7000) ),  -- Quader
            NULL, NULL, NULL, NULL)
  FROM PWMetaDaten m
  WHERE name = 'Fassade') t;

-- 12.4.3.1 Datenstrukturen und Tabellen

-- Metadatentabelle anlegen: 
CREATE TABLE TINMetaDaten (
  name     VARCHAR(50),                 -- Bezeichner
  tininfo  SDO_TIN,                     -- Metadaten
  CONSTRAINT pk_tinmt PRIMARY KEY(name) -- Primärschlüssel
);

-- Blocktabelle anlegen: 
CREATE TABLE TINBlockDaten AS
  SELECT * FROM MDSYS.SDO_TIN_BLK_TABLE;

-- Berechnen und Speichern einer Dreiecksvermaschung

-- Alte Daten löschen: 
DELETE FROM Quellpunkte;

-- Hier DGMOracle.sql ausführen.
COMMIT;

  DECLARE 
  tininfo     SDO_TIN;       -- das Metadaten-Objekt
BEGIN
  -- Metadaten-Objekt erzeugen:
  tininfo := SDO_TIN_PKG.INIT( 'TINMETADATEN', 'TININFO', 'TINBLOCKDATEN',
               'blk_capacity=400',
               SDO_GEOMETRY(2003, 25832, NULL, SDO_ELEM_INFO_ARRAY(1,1003,3),
                 SDO_ORDINATE_ARRAY(563300,5236000, 630600,5291067)),
               0.01, 3, NULL, NULL, NULL, NULL);
  -- Metadaten abspeichern: 
  INSERT INTO TINMetaDaten VALUES ('TINDaten',tininfo);
  -- Daten überführen: 
  SDO_TIN_PKG.CREATE_TIN (tininfo,'Quellpunkte');
END;
/

SELECT count(*), sum(NUM_TRIANGLES)
FROM TINBlockDaten;

-- 12.4.3.3 Anfragen

SELECT SDO_TIN_PKG.TO_GEOMETRY (points, triangles,
                                num_points, num_triangles, 2, 3, NULL)
FROM TINBlockDaten;

SELECT SDO_TIN_PKG.TO_GEOMETRY (t.points, t.triangles,
                                t.num_points, t.num_triangles, 2, 3, NULL)
FROM TABLE (
  SELECT SDO_TIN_PKG.CLIP_TIN (tininfo,                -- Metadaten-Objekt
            SDO_GEOMETRY(2003, 25832, NULL, SDO_ELEM_INFO_ARRAY(1,1003,3),
               SDO_ORDINATE_ARRAY(580000,5260000, 
                                  590000,5270000) ),  -- Anfragegeometrie
            NULL, NULL)
  FROM TINMetaDaten m
  WHERE name = 'TINDaten') t;

-- 12.4.3.4 Rasterbildung

DECLARE
  tin  SDO_TIN;          -- Dreiecksvermaschung
  geor SDO_GEORASTER;    -- Raster
BEGIN
  -- TIN holen, Raster anlegen und holen: 
  SELECT tininfo INTO tin FROM TINMetaDaten WHERE name = 'TINDaten';
  INSERT INTO GeoRasterkarten VALUES (50, 'TIN', SDO_GEOR.INIT('Raster',50));
  SELECT georaster INTO geor FROM GeoRasterkarten WHERE id = 50;
  -- Überführung: 
  SDO_TIN_PKG.TO_DEM(tin,geor,256,25);
  -- geändertes Raster abspeichern: 
  UPDATE GeoRasterkarten SET georaster = geor WHERE id = 50;
  COMMIT;
END;
/

-- Wertebereich der Höhen, die in den Rasterzellen gespeichert sind: 
SELECT SDO_GEOR.getRasterRange (gr.georaster)
FROM GeoRasterkarten gr
WHERE id = 50;

-- Höhenwert der Zelle (279/596) abfragen: 
SELECT SDO_GEOR.getCellValue (gr.georaster,0,279,596,0)
FROM GeoRasterkarten gr
WHERE id = 50;
