-- ----------------------
-- KAPITEL 13 (Oracle 21)
-- ----------------------

-- 13.1.4 Oracle Spatial

-- Quelltabelle anlegen und füllen:
CREATE TABLE GeoGemeinden1000ha AS
  SELECT * FROM GeoGemeinden
  WHERE hektar < 1000;

ALTER TABLE GeoGemeinden1000ha
  ADD CONSTRAINT pk_ggem1000 PRIMARY KEY(code);

-- Metadaten einfügen:
INSERT INTO USER_SDO_GEOM_METADATA
VALUES ('GEOGEMEINDEN1000HA','GEOM',
        SDO_DIM_ARRAY(SDO_DIM_ELEMENT('Länge',-180,180,1),
                      SDO_DIM_ELEMENT('Breite',-90,90,1)), 4326);
COMMIT;

-- R-Raum-Index anlegen:
CREATE INDEX GeoGemeinden1000ha_geom_ix ON GeoGemeinden1000ha(geom)
INDEXTYPE IS MDSYS.SPATIAL_INDEX_V2;

-- Tabelle für Clusterbeschreibungen anlegen:
CREATE TABLE Clusters (
  id   DECIMAL(3),                         -- ID des Clusters
  geo  SDO_GEOMETRY,                       -- Einzugsbereich des Clusters
  CONSTRAINT pk_cluster PRIMARY KEY (id)   -- Primärschlüssel
);

-- Metadaten einfügen:
INSERT INTO USER_SDO_GEOM_METADATA
VALUES ('CLUSTERS','GEO',
        SDO_DIM_ARRAY(SDO_DIM_ELEMENT('Länge',-180,180,1),
                      SDO_DIM_ELEMENT('Breite',-90,90,1)), 4326);
COMMIT;

-- Räumlichen Index anlegen:
CREATE INDEX Clusters_geo_ix ON Clusters(geo)
INDEXTYPE IS MDSYS.SPATIAL_INDEX_V2;

-- Clusterbeschreibungen berechnen und in die Tabelle einfügen: 
INSERT INTO Clusters (id,geo)
  SELECT *
  FROM TABLE(SDO_SAM.SPATIAL_CLUSTERS('GEOGEMEINDEN1000HA','GEOM',15));

COMMIT;

-- Funktion zur fehlerfreien Zuordnung zu Clustern:
CREATE OR REPLACE FUNCTION BIN_GEOMETRY (
  data_geom IN SDO_GEOMETRY, tolerance IN NUMBER, 
  cl_table IN VARCHAR2, cl_col IN VARCHAR2) RETURN NUMBER IS
BEGIN
  RETURN SDO_SAM.BIN_GEOMETRY (data_geom, tolerance, cl_table, cl_col);
EXCEPTION WHEN OTHERS THEN
  RETURN -1;
END;
/

SELECT clid, count(*), SDO_AGGR_UNION(SDOAGGRTYPE(geom,1e-6)) FROM
  (SELECT geom, BIN_GEOMETRY(geom,1,'CLUSTERS','GEO') AS clid
   FROM GeoGemeinden1000ha)
GROUP BY clid;


-- 13.2 Freiformkurven

CREATE TABLE KurvenDBLand (
  id    INTEGER,                             -- Identifikationsnummer
  name  VARCHAR(40),                         -- Bezeichnung
  geo   SDO_GEOMETRY,                        -- Kurvengeometrie
  CONSTRAINT pk_kurvenland PRIMARY KEY(id)   -- Primärschlüssel
);

INSERT INTO KurvenDBLand VALUES (0, 'Kontrollpunkte', 
  SDO_GEOMETRY(2005, NULL,NULL, SDO_ELEM_INFO_ARRAY(1,1,7),
  SDO_ORDINATE_ARRAY(3,1, 2,1, 5,7, 7,16, 11,12, 14,6, 18,4) ) );
INSERT INTO KurvenDBLand VALUES (1, 'Kurve Grad 1',
   SDO_GEOMETRY(2002, NULL,NULL, SDO_ELEM_INFO_ARRAY(1,2,3),
     SDO_ORDINATE_ARRAY(1, 7,       -- Grad und Anzahl der Kontrollpunkte
       3,1,1, 2,1,1, 5,7,1, 7,16,1, 11,12,1, 14,6,1, 18,4,1,
       9,        -- Anzahl der Knotenwerte = Anzahl Kontrollpunkte+Grad+1
       0, 0, 0, 0.25, 0.5, 0.75, 1, 1, 1) ) );
INSERT INTO KurvenDBLand VALUES (2, 'Kurve Grad 2',
   SDO_GEOMETRY(2002, NULL,NULL,
     SDO_ELEM_INFO_ARRAY(1,2,3),
     SDO_ORDINATE_ARRAY(2, 7,       -- Grad und Anzahl der Kontrollpunkte
       3,1,1, 2,1,1, 5,7,1, 7,16,1, 11,12,1, 14,6,1, 18,4,1,
       10,       -- Anzahl der Knotenwerte = Anzahl Kontrollpunkte+Grad+1
       0, 0, 0, 0.25, 0.4, 0.6, 0.75, 1, 1, 1) ) );
INSERT INTO KurvenDBLand VALUES (3, 'Kurve Grad 3',
   SDO_GEOMETRY(2002, NULL,NULL, SDO_ELEM_INFO_ARRAY(1,2,3),
     SDO_ORDINATE_ARRAY(3, 7,       -- Grad und Anzahl der Kontrollpunkte
       3,1,1, 2,1,1, 5,7,1, 7,16,1, 11,12,1, 14,6,1, 18,4,1,
       11,       -- Anzahl der Knotenwerte = Anzahl Kontrollpunkte+Grad+1
       0, 0, 0, 0, 0.25, 0.5, 0.75, 1, 1, 1, 1) ) );
INSERT INTO KurvenDBLand VALUES (4, 'Kurve Grad 3 gewichtet',
   SDO_GEOMETRY(2002, NULL,NULL, SDO_ELEM_INFO_ARRAY(1,2,3),
     SDO_ORDINATE_ARRAY(3, 7,       -- Grad und Anzahl der Kontrollpunkte
       3,1,1, 2,1,1, 5,7,3, 7,16,1, 11,12,3, 14,6,1, 18,4,1,
       11,       -- Anzahl der Knotenwerte = Anzahl Kontrollpunkte+Grad+1
       0, 0, 0, 0, 0.25, 0.5, 0.75, 1, 1, 1, 1) ) );
COMMIT;

SELECT k.geo.Get_WKT()
FROM KurvenDbLand k
WHERE id = 4

