-- ---------------------
-- KAPITEL 2 (Oracle 21)
-- ---------------------

-- 2.2.1 Datenmanipulation

-- Anlegen der Tabelle:
CREATE TABLE Gemeinden (
  gkz   DECIMAL(8),                    -- Gemeindeschlüssel
  name  VARCHAR(30)  NOT NULL,         -- Gemeindename
  einw  DECIMAL(7)   DEFAULT 0,        -- Einwohnerzahl
  CONSTRAINT pk_gem PRIMARY KEY (gkz)  -- Primärschlüssel
);

-- Ändern der Tabelle:
ALTER TABLE Gemeinden ADD CONSTRAINT ck_gem_einw CHECK (einw >= 0);

-- Löschen der Tabelle:
-- DROP TABLE Gemeinden;

-- 2.2.2.1 DML und DCL

-- Einfügen von Datensätze:
INSERT INTO Gemeinden (gkz,name,einw) VALUES (3241001,'Hannover',538068);
INSERT INTO Gemeinden (gkz,name) VALUES (3403000,'Oldenburg');
INSERT INTO Gemeinden VALUES (3451002,'Bad Zwischenahn',28896);
INSERT INTO Gemeinden VALUES (3451007,'Westerstede',22952);
INSERT INTO Gemeinden VALUES (3461004,'Elsfleth',9114);
INSERT INTO Gemeinden VALUES (3404000,'Osnabrück',165251);
INSERT INTO Gemeinden VALUES (3405000,'Wilhelmshaven',76089);
COMMIT;

-- Ändern vorhandener Datensätze:
UPDATE Gemeinden SET einw = 169077 WHERE gkz = 3403000;
UPDATE Gemeinden SET einw = einw - 1143 WHERE Name = 'Hannover';
COMMIT;

-- Löschen von Datensätzen und Rücknahme des Löschens:
DELETE FROM Gemeinden WHERE name = 'Hannover';
ROLLBACK;

-- 2.1.2.2 Erzeugung von Schlüsselwerten

-- Anlegen der Tabelle Kreise:
CREATE TABLE Kreise (
  id         INTEGER  GENERATED ALWAYS AS IDENTITY (START WITH 1),
                                              -- generierter Schlüssel
  name       VARCHAR(50)  NOT NULL,           -- Name des Kreises
  qkm        DECIMAL(8,2),                    -- Fläche in qkm
  kreisstadt DECIMAL(8),                      -- GKZ der Kreisstadt
  CONSTRAINT pk_id PRIMARY KEY (id),          -- Primärschlüssel
  CONSTRAINT ck_krs_qkm CHECK (qkm > 0)       -- Integritätsregel
);

-- Einfügen von Tupeln in die Tabelle Kreise:
INSERT INTO Kreise (name,qkm,kreisstadt)
  VALUES ('Oldenburg Stadt',102.97,3403000);
INSERT INTO Kreise (name,qkm,kreisstadt)
  VALUES ('Ammerland',728.16,3451007);
INSERT INTO Kreise (name,qkm,kreisstadt)
  VALUES ('Osnabrück Stadt',119.80,3404000);
INSERT INTO Kreise (name,qkm,kreisstadt)
  VALUES ('Osnabrück Land',2121.56,3404000);
COMMIT;

-- 2.1.3 Beziehungen

-- Ändern der Tabelle:
ALTER TABLE Kreise ADD CONSTRAINT fk_krs_kst FOREIGN KEY (kreisstadt)
  REFERENCES Gemeinden(gkz)                         -- Fremdschlüssel
  
-- Anlegen der Tabelle Hochschulen:
CREATE TABLE Hochschulen (
  num    INTEGER,                      -- Hochschulschlüssel
  name   VARCHAR(20)  NOT NULL,        -- Hochschulname
  stud   DECIMAL(6),                   -- Zahl der Studierenden
  CONSTRAINT pk_hs PRIMARY KEY (num)   -- Primärschlüssel
);

-- Füllen der Tabelle:
INSERT INTO Hochschulen VALUES (1,'Uni Oldenburg',14445);
INSERT INTO Hochschulen VALUES (2,'Jade Hochschule',6639);
INSERT INTO Hochschulen VALUES (3,'Uni Hannover',28141);

-- Anlegen der Vernüpfungstabelle HatStandortIn:
CREATE TABLE HatStandortIn (
  hs        INTEGER,                                -- Hochschulschlüssel
  gemeinde  DECIMAL(8),                             -- GKZ des Standorts
  CONSTRAINT pk_stdort PRIMARY KEY (hs, gemeinde),  -- komb. Primärschlüssel
  CONSTRAINT fk_stdort_hs FOREIGN KEY (hs)
    REFERENCES Hochschulen(num) ON DELETE CASCADE,  -- 1. Fremdschlüssel
  CONSTRAINT fk_stdort_gem FOREIGN KEY (gemeinde)
    REFERENCES Gemeinden(gkz) ON DELETE CASCADE     -- 2. Fremdschlüssel
);

-- Einfügen der Datensätze in HatStandortIn:
INSERT INTO HatStandortIn VALUES (3,3241001);
INSERT INTO HatStandortIn VALUES (1,3403000);
INSERT INTO HatStandortIn VALUES (2,3403000);
INSERT INTO HatStandortIn VALUES (2,3461004);
INSERT INTO HatStandortIn VALUES (2,3405000);
COMMIT;

-- 2.1.4.1 Einfache Anfragen

-- keine Projektion = Ausgabe aller Attribute:
SELECT * FROM Gemeinden;

-- Projektion und Umbenennung:
SELECT name, einw AS einwohner FROM Gemeinden;

-- Auswahl der Großstädte:
SELECT * FROM Gemeinden WHERE einw >= 100000;

-- Gemeinden, deren Name mit W beginnt oder mit weniger als 50000 Einw.:
SELECT name, einw
FROM Gemeinden
WHERE name LIKE 'W%' OR einw < 50000;

-- aufsteigende Sortierung nach dem Namen:
SELECT gkz, name FROM Gemeinden ORDER BY name;

-- absteigende Sortierung nach der Einwohnerzahl:
SELECT gkz, name FROM Gemeinden ORDER BY einw DESC;

-- Duplikate entfernen:
SELECT DISTINCT hs FROM HatStandortIn;

-- 2.1.4.2 Verbindende Anfragen

-- Die Kreisnamen mit den Namen der zugehörigen Kreisstädte:
SELECT Kreise.name, Gemeinden.name
FROM Kreise, Gemeinden
WHERE kreisstadt = gkz;

-- Verbund nach SQL-92:
SELECT Kreise.name, Gemeinden.name
FROM Kreise INNER JOIN Gemeinden
ON kreisstadt = gkz;

-- Äußerer Verbund:
SELECT Gemeinden.name, Kreise.name
FROM Gemeinden LEFT JOIN Kreise
ON gkz = kreisstadt;

-- Kartesisches Produkt:
SELECT Kreise.name, Gemeinden.name FROM Kreise, Gemeinden;
-- Kartesisches Produkt gemäß SQL-92:
SELECT Kreise.name, Gemeinden.name FROM Kreise CROSS JOIN Gemeinden;

-- Unteranfrage:
SELECT k.name AS kreis, g.name AS gemeinde
FROM Kreise k INNER JOIN (SELECT * FROM Gemeinden WHERE einw >= 100000) g
ON kreisstadt = gkz;

-- Fehlerhafte Unterabfrage: 
UPDATE Gemeinden SET einw = (SELECT num FROM Hochschulen)
WHERE name = 'Bad Zwischenahn';

-- IN-Bedingung:
SELECT name
FROM Gemeinden
WHERE gkz IN (SELECT kreisstadt FROM Kreise);

-- 2.1.4.3 Gruppierung und Aggregation

-- Anzahl der Gemeinden:
SELECT count(*) FROM Gemeinden;

-- Durchschnittliche Einwohnerzahl der größeren Gemeinden:
SELECT avg(einw) FROM Gemeinden WHERE einw > 50000;

-- Gruppierung:
SELECT gemeinde, count(hs)  -- der direkte Zugriff auf hs ist nicht zulässig
FROM HatStandortIn
GROUP BY gemeinde;

-- Gruppierung mit HAVING-Klausel:
SELECT gemeinde, count(hs)
FROM HatStandortIn
GROUP BY gemeinde
HAVING count(hs) > 1;

-- Window-Funktion:
SELECT gkz, name, einw, 100*einw/max(einw) OVER () AS proz FROM Gemeinden;

-- Window-Funktion mit PARTITION BY:
SELECT gkz, name, einw, floor(log(10,einw)) as klasse,
       avg(einw) OVER (PARTITION BY floor(log(10,einw)))
FROM Gemeinden;

-- 2.1.4.4 Sichten

-- Anlegen der Sicht:
CREATE VIEW Grossstädte AS 
  SELECT * FROM Gemeinden WHERE einw >= 100000;
-- Verwenden der Sicht:
SELECT name, einw FROM Grossstädte;

-- Index anlegen:
CREATE INDEX Gemeinden_name_ix ON Gemeinden(name);

-- Index wieder löschen:
DROP INDEX Gemeinden_name_ix;


-- 2.3.2.1 Datentypen

-- Umwandlung von gkz in eine Zeichenkette und Bestimmung von deren Länge: 
SELECT Length( CAST(gkz AS VARCHAR(80)) ) FROM Gemeinden;
SELECT Length( gkz ) FROM Gemeinden;

-- 2.3.2.2 Tabelle Dual

SELECT sysdate FROM Dual;

-- 2.3.2.3 RowID

SELECT rowid, gkz, name FROM Gemeinden;

-- 2.3.3.4 Large Objects (LOBs)

-- Tabelle zur Speicherung von Rasterkarten in BLOBs anlegen:
CREATE TABLE Rasterkarten (
  id         INTEGER,                 -- ID
  name       VARCHAR(80)  NOT NULL,   -- Kartenname
  karte      BLOB,                    -- Rasterkarte als BLOB
  worldfile  VARCHAR(255),            -- Georeferenzierung
  CONSTRAINT pk_rk PRIMARY KEY (id)   -- Primärschlüssel
)
LOB (karte) STORE AS ( CHUNK 16K );

-- 2.3.3.5 JSON

CREATE TABLE GeoDbLandJson (
   id   DECIMAL(4),
   doc  JSON        NOT NULL,
   CONSTRAINT GeoDbLandJson_pk PRIMARY KEY(id)
);

INSERT INTO GeoDbLandJson VALUES (1,
'{"id": 1,
  "name": "Flaggenmast", 
  "height": 10.5,
  "geo": {"type": "Point", "coordinates": [10,11]}}');
INSERT INTO GeoDbLandJson VALUES (2,
'{"id": 2,
  "name": "Steg", 
  "geo": {"type": "LineString", "coordinates": [[4,4],[5,7],[8,8]]}}');
INSERT INTO GeoDbLandJson VALUES (3,
'{"id": 3,
  "name": "Baumreihe",
  "species": ["Eiche", "Buche", "Eiche"],
  "geo": {"type": "MultiPoint", "coordinates": [[10,19],[12,19],[14,19]]},
  "photo": {"name": "Baumreihe", "file": "img64774.jpg", "type": "jpg",
            "date": "2020-10-27"}}');
INSERT INTO GeoDbLandJson VALUES (4,
'{"id": 4,
  "name": "Wall",
  "species": ["Kirschlorbeer", "Thuja"],
  "geo": {"type": "LineString", "coordinates":
          [[1,15],[2.414,15.586],[3,17],[3.586,18.414],[5,19],[7,19]]}}');
INSERT INTO GeoDbLandJson VALUES (5,
'{"id": 5,
  "name": "Haus",
  "geo": {"type": "Polygon", "coordinates":
          [[[9,9],[12,9],[12,10],[9,10],[9,9]]]}}');
INSERT INTO GeoDbLandJson VALUES (6,
'{"id": 6,
  "name": "Land",
  "geo": {"type": "MultiPolygon", "coordinates":
          [[[[1,1],[19,1],[19,20],[1,20],[1,1]],
            [[4,5],[3,10],[7,14],[13,15],[17,12],[16,8],[13,5],[9,3],[4,5]]],
           [[[7,7],[12,7],[13,11],[10,13],[7,10],[7,7]]]]}}');

SELECT id, json_serialize(doc) FROM GeoDbLandJson;
SELECT id, json_serialize(doc RETURNING CLOB) FROM GeoDbLandJson;

SELECT id, j.doc.name FROM GeoDbLandJson j;
SELECT id, j.doc.species[0] FROM GeoDbLandJson j;
SELECT id, j.doc.photo."date" FROM GeoDbLandJson j;

SELECT id, j.doc.name.string() FROM GeoDbLandJson j;
SELECT id, j.doc.id.number() FROM GeoDbLandJson j;
SELECT id, j.doc.photo."date".date() FROM GeoDbLandJson j;

SELECT id, json_query(doc,'$.name') FROM GeoDbLandJson;
SELECT id, json_query(doc,'$.name.string()') FROM GeoDbLandJson;
SELECT id, json_value(doc,'$.name') FROM GeoDbLandJson;
SELECT id, json_value(doc,'$.id' RETURNING NUMBER) FROM GeoDbLandJson;

SELECT id FROM GeoDbLandJson j WHERE j.doc.name.string() = 'Baumreihe';
SELECT id FROM GeoDbLandJson j WHERE j.doc.id.number() = 5;

-- 2.3.2.6 Anfragepläne

EXPLAIN PLAN FOR 
  SELECT * FROM Gemeinden WHERE gkz = '3403000';
  
  
-- 2.3.3 Prozedurale Erweiterungen

-- Funktion anlegen:
CREATE OR REPLACE FUNCTION kreisstadttext (name VARCHAR) RETURN VARCHAR IS
BEGIN
  IF name IS NULL THEN
    RETURN 'keine Kreisstadt';
  ELSE
    RETURN name;
  END IF;
END;
/

-- Funktion aufrufen:
SELECT Gemeinden.name, kreisstadttext(Kreise.name)
FROM Gemeinden LEFT JOIN Kreise
ON gkz = kreisstadt;

-- Log-Tabelle anlegen:
CREATE TABLE LogHochschulen (
  hs     INTEGER,     -- Hochschulnummer
  stud   DECIMAL(6),  -- Anzahl Studierende
  datum  TIMESTAMP    -- Einfügezeitpunkt
);

-- PL/SQL-Block ausführen:
DECLARE
  -- Deklaration Cursor und Variable
  CURSOR c IS SELECT * FROM Hochschulen;   -- DB-Cursor
  akt_t c%ROWTYPE;                  -- aktuelle Antwort
BEGIN
  -- alle Antworten durchlaufen
  FOR akt_t IN c LOOP
    -- Datensatz einfügen
    INSERT INTO LogHochschulen VALUES (akt_t.num, akt_t.stud, sysdate);
  END LOOP;
END;
/

-- Tabelle ausgeben:
SELECT * FROM LogHochschulen;

-- Datensätze löschen:
ROLLBACK;

-- Benannte Prozedur deklarieren:
CREATE OR REPLACE PROCEDURE fuelleLogHS IS
  -- Deklaration Cursor und Variable
  CURSOR c IS SELECT * FROM Hochschulen;   -- DB-Cursor
  akt_t c%ROWTYPE;                  -- aktuelle Antwort
BEGIN
  -- alle Antworten durchlaufen
  FOR akt_t IN c LOOP
    -- Datensatz einfügen
    INSERT INTO LogHochschulen VALUES (akt_t.num, akt_t.stud, sysdate);
  END LOOP;
END;
/

-- Prozedur aufrufen, Commit:
CALL fuelleLogHS();
COMMIT;

DECLARE
  studNum  DECIMAL(6);   -- Anzahl der Studenten
BEGIN
  SELECT stud INTO studNum FROM Hochschulen WHERE num = 3 FOR UPDATE;
    -- der Datensatz ist gesperrt, s.d. eine komplexe Aktion folgen könnte
  studNum := studNum - 500;
  UPDATE Hochschulen SET stud = studNum WHERE num = 3;
  COMMIT;
END;
/

-- 2.3.4.1 Felder

CREATE OR REPLACE TYPE TextFeld AS
VARRAY(3) OF VARCHAR(20);
/

-- Erweiterung der Tabelle "Gemeinden":
ALTER TABLE Gemeinden
ADD inhalt TextFeld;

UPDATE Gemeinden
SET inhalt = TextFeld('Marktplatz', 'Schloss', 'Lange Straße')
WHERE gkz = 3403000;

COMMIT;

-- Nutzung:
SELECT gkz, name, inhalt
FROM Gemeinden
WHERE inhalt IS NOT NULL;         -- Prüfung, ob kein Nullwert

ALTER TYPE TextFeld MODIFY LIMIT 10 CASCADE;

CREATE OR REPLACE FUNCTION elementAusTextFeld
                           (i DECIMAL, feld TextFeld) RETURN VARCHAR IS
BEGIN
  IF feld IS NULL THEN
    RETURN '/';
  ELSIF (i >= 1) AND (i <= feld.count) THEN
    RETURN feld(i);
  ELSE
    RETURN '-';
  END IF;
END;
/

-- Verwendung der Zugriffsfunktion:
SELECT gkz, name
FROM Gemeinden
WHERE elementAusTextFeld(2,inhalt) = 'Schloss';

SELECT g.gkz, t.*
FROM Gemeinden g, TABLE(g.inhalt) t
WHERE COLUMN_VALUE <> 'Schloss';

-- an alle nichtleeren Felder ein zusätzliches Element anfügen:
DECLARE
  CURSOR c IS SELECT inhalt FROM GemeindeVideos
              WHERE inhalt IS NOT NULL FOR UPDATE;  -- DB-Cursor
  akt_t c%ROWTYPE;                                  -- aktuelle Antwort
BEGIN
  FOR akt_t IN c LOOP
    IF akt_t.inhalt.count < akt_t.inhalt.limit THEN
      akt_t.inhalt.extend(1);
      akt_t.inhalt(akt_t.inhalt.count) := 'Bahnhof';
      UPDATE GemeindeVideos SET inhalt = akt_t.inhalt WHERE CURRENT OF c;
    END IF;
  END LOOP;
  COMMIT;
END;
/

-- Ergebnis anzeigen:
SELECT inhalt FROM GemeindeVideos WHERE inhalt IS NOT NULL;

-- 2.3.4.2 Klassen

CREATE OR REPLACE TYPE Punkt2D_Typ AS OBJECT (
  x  DECIMAL(10,7),       -- x-Koordinate in Grad
  y  DECIMAL(9,7)         -- y-Koordinate in Grad
) NOT FINAL
/

-- 2.3.4.3 Verwendung als Spaltenobjekte

-- Tabelle erweitern:
ALTER TABLE Gemeinden ADD centrum Punkt2D_Typ;

-- Attribut setzen:
UPDATE Gemeinden SET centrum=Punkt2D_Typ(9.7358,52.3803) WHERE gkz = 3241001;
UPDATE Gemeinden SET centrum=Punkt2D_Typ(8.2275,53.1375) WHERE gkz = 3403000;
UPDATE Gemeinden SET centrum=Punkt2D_Typ(7.9981,53.1844) WHERE gkz = 3451002;
UPDATE Gemeinden SET centrum=Punkt2D_Typ(7.9256,53.2581) WHERE gkz = 3451007;
UPDATE Gemeinden SET centrum=Punkt2D_Typ(8.4573,53.2372) WHERE gkz = 3461004;
UPDATE Gemeinden SET centrum=Punkt2D_Typ(8.0450,52.2810) WHERE gkz = 3404000;
UPDATE Gemeinden SET centrum=Punkt2D_Typ(8.1081,53.5397) WHERE gkz = 3405000;
COMMIT;

SELECT gkz, name, centrum, g.centrum.x, (centrum).y
FROM Gemeinden;

ALTER TYPE Punkt2D_Typ REPLACE AS OBJECT (
  x  DECIMAL(10,7),       -- x-Koordinate in Grad
  y  DECIMAL(9,7),        -- y-Koordinate in Grad
  MEMBER FUNCTION istWestlichVon (vgl Punkt2D_Typ) RETURN INTEGER,
  -- Testet, ob das Punktobjekt westlich vom Punkt vgl liegt.
  -- Falls ja, wird 1, sonst 0 zurückgegeben.
  MEMBER FUNCTION gebeBeschreibung RETURN VARCHAR,
  -- Gibt vom Objekt eine Beschreibung als String zurück.
  STATIC FUNCTION nullpkt RETURN Punkt2D_Typ
  -- Gibt einen Nullpunkt zurück.
) NOT FINAL;

CREATE OR REPLACE TYPE BODY Punkt2D_Typ AS
  MEMBER FUNCTION istWestlichVon (vgl Punkt2D_Typ) RETURN INTEGER IS
    BEGIN
      IF x < vgl.x THEN   -- entspricht: IF SELF.x < vgl.x THEN
        RETURN 1;
      ELSE
        RETURN 0;
      END IF;
    END;
  MEMBER FUNCTION gebeBeschreibung RETURN VARCHAR IS
    BEGIN
      RETURN '(' || x || ' , ' || y || ')';
    END;
  STATIC FUNCTION nullpkt RETURN Punkt2D_Typ IS
    BEGIN
      RETURN Punkt2D_Typ(0,0);
    END;
END;
/

SELECT name, g.centrum.gebeBeschreibung(), (centrum).gebeBeschreibung()
FROM Gemeinden g;

SELECT g2.name, g2.centrum.gebeBeschreibung()
FROM Gemeinden g1 INNER JOIN Gemeinden g2
ON g1.gkz = 3403000 
WHERE g2.centrum.istWestlichVon(g1.centrum) = 1;

SELECT name
FROM Gemeinden
WHERE (centrum).istWestlichVon(Punkt2D_Typ.nullpkt()) = 0;

-- 2.3.4.4 Objekttabellen

CREATE TABLE Messpunkte OF Punkt2D_Typ;

INSERT INTO Messpunkte VALUES (9.6351, 52.3799);
INSERT INTO Messpunkte VALUES (9.6410, 52.3813);
INSERT INTO Messpunkte VALUES (9.6359, 52.3795);
COMMIT;

-- Attributwerte abfragen:
SELECT * FROM Messpunkte;

-- Objekte abfragen:
SELECT VALUE(m) FROM Messpunkte m;

-- Objekte abfragen (alternativ):
SELECT object_value FROM Messpunkte;

-- OID abfragen:
SELECT object_id FROM Messpunkte;

-- Objektreferenz REF abfragen:
SELECT REF(m) FROM Messpunkte m;

-- Tabelle anlegen:
CREATE TABLE Messungen (
  datum      DATE,                                -- Datumsangabe
  temperatur DECIMAL(5,2),                        -- Temperaturangabe
  ort        REF Punkt2D_Typ SCOPE IS Messpunkte  -- Referenz auf Messpunkt
);

-- Datensätze einfügen:
INSERT INTO Messungen
VALUES ('17.07.2021', 7.3,
        (SELECT REF(m) FROM Messpunkte m WHERE m.x = 9.6351) );
COMMIT;

SELECT datum, temperatur, DEREF(ort)
FROM Messungen;

SELECT datum, temperatur, (ort).x, (ort).y
FROM Messungen;

-- 2.3.4.5 Vererbung und Polymorphie

CREATE OR REPLACE TYPE BenannterPunkt2D_Typ UNDER Punkt2D_Typ (
  name VARCHAR(30),            -- der Name
  OVERRIDING MEMBER FUNCTION gebeBeschreibung RETURN VARCHAR,
  -- Gibt vom Objekt eine Beschreibung als String zurück.
  MEMBER FUNCTION hatGleichenNamen (p BenannterPunkt2D_Typ)
         RETURN INTEGER,
  -- Testet, ob das Punktobjekt den gleichen Namen wie p hat.
  -- Falls ja, wird 1, sonst 0 zurückgegeben.
) NOT FINAL;
/

CREATE OR REPLACE TYPE BODY BenannterPunkt2D_Typ AS
  OVERRIDING MEMBER FUNCTION gebeBeschreibung RETURN VARCHAR IS
    BEGIN
      RETURN name || ': ' || '(' || x || ' , ' || y || ')' ;
    END;
  MEMBER FUNCTION hatGleichenNamen (p BenannterPunkt2D_Typ)
                  RETURN INTEGER IS
    BEGIN
      IF name = p.name THEN
        RETURN 1;
      ELSE
        RETURN 0;
      END IF;
    END;
END;
/

-- Daten einfügen bzw. ändern:
INSERT INTO Gemeinden (gkz,name,einw,centrum)
VALUES (4011000, 'Bremen', 540950,
        BenannterPunkt2D_Typ(8.8139, 53.0839, 'Marktplatz') );
		
UPDATE Gemeinden
SET centrum = BenannterPunkt2D_Typ(8.2275, 53.1375, 'Schlossplatz')
WHERE gkz = 3403000;

UPDATE Gemeinden
SET centrum =  BenannterPunkt2D_Typ(7.9256, 53.2581, 'Marktplatz')
WHERE gkz = 3451007;
COMMIT;

-- Daten abfragen:
SELECT name, g.centrum.gebeBeschreibung()
FROM Gemeinden g;

SELECT g1.name, g2.name, 
       TREAT(g1.centrum AS BenannterPunkt2D_Typ).hatGleichenNamen(
                               TREAT(g2.centrum AS BenannterPunkt2D_Typ) )
FROM Gemeinden g1 INNER JOIN Gemeinden g2
ON g1.gkz < g2.gkz
WHERE g1.centrum IS OF(BenannterPunkt2D_Typ) AND 
      g2.centrum IS OF(BenannterPunkt2D_Typ);
