-- ---------------------
-- KAPITEL 4 (Oracle 21)
-- ---------------------

-- 4.2 Oracle Spatial

SELECT sdo_version FROM Dual; 

-- 4.2.1.1 Deklaration und Konstruktoren

CREATE TABLE GeoDbLand (
  id     INTEGER,                         -- ID
  name   VARCHAR(20),                     -- Objektname
  geo    SDO_GEOMETRY,                    -- die Geometrie
  CONSTRAINT pk_gdbland PRIMARY KEY(id)   -- Primärschlüssel
);

-- Flaggenmast einfügen:
INSERT INTO GeoDbLand (id,name,geo)
VALUES (1, 'Flaggenmast', SDO_GEOMETRY(2001,NULL,
                              SDO_POINT_TYPE(10,11,NULL),NULL,NULL));
COMMIT;

-- Tabelle anzeigen:
SELECT * FROM GeoDbLand;

INSERT INTO GeoDbLand (id,name,geo)
VALUES (2,'Steg', SDO_GEOMETRY(2002, NULL,NULL,
        SDO_ELEM_INFO_ARRAY(1,2,1), SDO_ORDINATE_ARRAY(4,4, 5,7, 8,8)));
INSERT INTO GeoDbLand (id,name,geo)
VALUES (3,'Baumreihe', SDO_GEOMETRY(2005, NULL,NULL,
        SDO_ELEM_INFO_ARRAY(1,1,3),
        SDO_ORDINATE_ARRAY(10,19, 12,19, 14,19)));
INSERT INTO GeoDbLand (id,name,geo)
VALUES (4,'Wall', SDO_GEOMETRY(2002, NULL,NULL,
        SDO_ELEM_INFO_ARRAY(1,2,2),
        SDO_ORDINATE_ARRAY(1,15, 2.414,15.586, 3,17, 3.586,18.414, 5,19)));
COMMIT;

INSERT INTO GeoDbLand (id,name,geo)
VALUES (5, 'Haus', SDO_GEOMETRY(2003, NULL,NULL,
                     SDO_ELEM_INFO_ARRAY(1,1003,3),
                     SDO_ORDINATE_ARRAY(9,9, 12,10)));
					 
UPDATE GeoDbLand SET geo = SDO_GEOMETRY(2003, NULL,NULL,
                     SDO_ELEM_INFO_ARRAY(1,1003,1),
                     SDO_ORDINATE_ARRAY(9,9, 12,9, 12,10, 9,10, 9,9))
WHERE name = 'Haus';
COMMIT;

INSERT INTO GeoDbLand (id,name,geo)
VALUES (6, 'Land', SDO_GEOMETRY(2007,NULL,NULL,
               SDO_ELEM_INFO_ARRAY(1,1003,1, 11,2003,1, 29,1003,1),
               SDO_ORDINATE_ARRAY(1,1, 19,1, 19,20, 1,20, 1,1,
                 4,5, 3,10, 7,14, 13,15, 17,12, 16,8, 13,5, 9,3, 4,5,
                 7,7, 12,7, 13,11, 10,13, 7,10, 7,7)));
COMMIT;

UPDATE GeoDbLand
SET geo = SDO_GEOMETRY(2002, NULL,NULL,
                        SDO_ELEM_INFO_ARRAY(1,4,2, 1,2,2, 9,2,1),
                        SDO_ORDINATE_ARRAY(1,15, 2.414,15.586, 3,17,
                                           3.586,18.414, 5,19, 7,19))
WHERE name = 'Wall';
COMMIT;

SELECT SDO_GEOMETRY('LINESTRING(4 4,5 7,8 8)') FROM Dual;

SELECT SDO_UTIL.FROM_GEOJSON(j.doc.geo) FROM GeoDbLandJson j;

SELECT json_value(doc,'$.geo' RETURNING SDO_GEOMETRY) FROM GeoDbLandJson;

-- 4.2.1.2 Informationsschema

INSERT INTO USER_SDO_GEOM_METADATA
VALUES ('GEODBLAND', 'GEO', SDO_DIM_ARRAY(
                              SDO_DIM_ELEMENT('X', 0,20, 0.001),
                              SDO_DIM_ELEMENT('Y', 0,20, 0.001) ), null);
COMMIT;

SELECT DISTINCT *
FROM SDO_DIST_UNITS
WHERE conversion_factor >= 0.3 AND conversion_factor <= 1 AND
      LENGTH(sdo_unit) <= 5
ORDER BY sdo_unit;

SELECT DISTINCT *
FROM SDO_AREA_UNITS
WHERE conversion_factor >= 0.1 AND conversion_factor <= 10 AND
      LENGTH(sdo_unit) <= 8
ORDER BY sdo_unit;

SELECT DISTINCT (geo).Get_Dims() FROM GeoDbLand;

SELECT DISTINCT (geo).ST_CoordDim() FROM GeoDbLand;

SELECT name, (geo).Get_GType() FROM GeoDbLand;

SELECT (geo).Get_WKT()     -- TO_CHAR((geo).Get_WKT()) für einen VARCHAR
FROM GeoDbLand;

SELECT name, SDO_UTIL.TO_GML311GEOMETRY(geo) FROM GeoDbLand;

SELECT name, (geo).Get_GeoJson()        -- oder: SDO_UTIL.TO_GEOJSON(geo) 
FROM GeoDbLand WHERE id = 1 OR id = 2;

SELECT id, name, SDO_UTIL.GETVERTICES(geo)
FROM (SELECT * FROM GeoDbLand WHERE id > 1)
WHERE SDO_UTIL.GETNUMVERTICES(geo) > 10;

-- Zugriff auf alle Punkte in GeoDbLand: 
SELECT SDO_GEOMETRY(2001,NULL,SDO_POINT_TYPE(t.x,t.y,NULL),NULL,NULL)
FROM GeoDbLand, TABLE(SDO_UTIL.GETVERTICES(geo)) t;

-- Zugriff auf einen bestimmten Punkt in GeoDbLand: 
SELECT g.id, name, t.x, t.y
FROM (SELECT * FROM GeoDbLand WHERE id > 1) g,
     TABLE(SDO_UTIL.GETVERTICES(g.geo)) t
WHERE SDO_UTIL.GETNUMVERTICES(g.geo) > 10 AND t.id = 3;

-- Zugriff auf Start und Endpunkt: 
SELECT name, (s).x, (s).y, (e).x, (e).y FROM
  (SELECT name, SDO_UTIL.GETFIRSTVERTEX(geo) AS s, 
                SDO_UTIL.GETLASTVERTEX(geo) AS e
   FROM GeoDbLand WHERE (geo).Get_GType() = 2);
   
SELECT SDO_UTIL.EXTRACT(geo,1)
FROM GeoDbLand WHERE Name = 'Land';

SELECT SDO_UTIL.EXTRACT(geo,1,2)
FROM GeoDbLand WHERE Name = 'Land';

SELECT name, SDO_UTIL.GETNUMELEM(geo) FROM GeoDbLand;

SELECT name, SDO_UTIL.POLYGONTOLINE(geo) FROM GeoDbLand;

SELECT SDO_GEOM.SDO_ARC_DENSIFY (geo,0.001,'arc_tolerance=0.01')
FROM GeoDbLand
WHERE name = 'Wall';

-- 4.2.1.4 Validieren und Korrigieren der Geometrie

SELECT id FROM GeoDbLand
WHERE SDO_GEOM.VALIDATE_GEOMETRY_WITH_CONTEXT(geo,0.001) <> 'TRUE';

SELECT id, SDO_GEOM.VALIDATE_GEOMETRY_WITH_CONTEXT(geo,m.diminfo)
FROM GeoDbLand, USER_SDO_GEOM_METADATA m
WHERE m.table_name = 'GEODBLAND' AND m.column_name = 'GEO';

-- Fehlerhafte Geometrie einfügen:
INSERT INTO GeoDbLand (id,name,geo)
VALUES (7, 'Haus 2', SDO_GEOMETRY(2003, NULL,NULL,
                       SDO_ELEM_INFO_ARRAY(1,1003,1),
                       SDO_ORDINATE_ARRAY(9,9, 9,10, 12,10, 12,9, 9,9)));

UPDATE GeoDbLand
SET geo = SDO_GEOM.SDO_SELF_UNION (geo,0.001)
WHERE name = 'Haus 2';

SELECT SDO_GEOM.SDO_SELF_UNION( SDO_GEOMETRY(2003, NULL,NULL,
         SDO_ELEM_INFO_ARRAY(1,1003,1),
         SDO_ORDINATE_ARRAY(9,9, 9,11, 8,10, 12,10, 12,9, 9,9)), 0.001)
FROM DUAL;

UPDATE GeoDbLand
SET geo = SDO_MIGRATE.TO_CURRENT (geo,
            SDO_DIM_ARRAY( SDO_DIM_ELEMENT('X', 0,20, 0.001),
                           SDO_DIM_ELEMENT('Y', 0,20, 0.001)) )
WHERE name = 'Haus 2';

-- Haus 2 wieder löschen: 
ROLLBACK;

SELECT SDO_UTIL.REMOVE_DUPLICATE_VERTICES( 
         SDO_GEOMETRY(2003, NULL,NULL,
         SDO_ELEM_INFO_ARRAY(1,1003,1),
         SDO_ORDINATE_ARRAY(9,9, 12,9, 12.001,9, 12,10, 9,10, 9,9)), 0.01)
FROM Dual;

-- 4.2.1.5 Geometrische Funktionen

SELECT g1.name, g2.name, SDO_GEOM.SDO_DISTANCE(g1.geo,g2.geo,0.001),
                         SDO_GEOM.WITHIN_DISTANCE(g1.geo,5,g2.geo,0.001)
FROM GeoDbLand g1 INNER JOIN GeoDbLand g2
ON g1.id < g2.id
ORDER BY g1.id, g2.id;

SELECT name, SDO_GEOM.SDO_LENGTH(geo,0.001)
FROM GeoDbLand;

SELECT name, SDO_GEOM.SDO_AREA(geo,0.001)
FROM GeoDbLand
WHERE (geo).Get_GType() = 3 OR (geo).Get_GType() = 7;

SELECT name, SDO_GEOM.SDO_CENTROID(geo,0.001)
FROM GeoDbLand
WHERE SDO_GEOM.SDO_CENTROID(geo,0.001) IS NOT NULL;

SELECT name, SDO_GEOM.SDO_POINTONSURFACE(geo,0.001)
FROM GeoDbLand;

SELECT name, SDO_UTIL.INTERIOR_POINT(geo,0.001)
FROM GeoDbLand
WHERE name = 'Haus' OR name = 'Land';

-- minimal umgebendes Rechteck (MUR)
SELECT SDO_GEOM_MBR(geo) FROM GeoDbLand WHERE name = 'Wall';

-- konvexe Hülle: 
SELECT SDO_GEOM.SDO_CONVEXHULL(geo,0.01) FROM GeoDbLand WHERE name = 'Steg';

SELECT geo, SDO_GEOM.SDO_BUFFER(geo,2,0.001)
FROM GeoDbLand WHERE name <> 'Land';

SELECT geo, SDO_GEOM.SDO_BUFFER(geo,-0.5,0.001)
FROM GeoDbLand WHERE name = 'Land';

SELECT SDO_GEOM.SDO_DIFFERENCE(g1.geo,g2.geo,0.001)
FROM GeoDbLand g1, GeoDbLand g2
WHERE g1.name = 'Land' AND g2.name = 'Haus';

-- 4.2.1.6 Prüfung topologischer Prädikate

SELECT SDO_GEOM.RELATE(g1.geo,'ANYINTERACT',g2.geo,0.001) AS ANYINTERACT,
       SDO_GEOM.RELATE(g1.geo,'INSIDE+EQUAL',g2.geo,0.001) AS INSIDE_EQ,
       SDO_GEOM.RELATE(g1.geo,'TOUCH',g2.geo,0.001) AS TOUCH,
       SDO_GEOM.RELATE(g1.geo,'DETERMINE',g2.geo,0.001) AS DETERMINE
FROM GeoDbLand g1, GeoDbLand g2
WHERE g1.name = 'Haus' AND g2.name = 'Land';

-- 4.2.1.7 Koordinatenbezugssysteme

-- Ausgabe ausgewählter Koordinatenbezugssysteme:
SELECT srid, coord_ref_sys_name, data_source AS SRC, coord_ref_sys_kind
FROM SDO_COORD_REF_SYS
WHERE srid IN (2056, 3416, 4326, 25832, 31467, 262148, 900913);

-- Gruppierung der Bezugssysteme nach Kategorie:
SELECT coord_ref_sys_kind, count(*)
FROM SDO_COORD_REF_SYS
GROUP BY coord_ref_sys_kind ORDER BY coord_ref_sys_kind;

-- Standardmaßeinheit für ausgewählte projizierte Bezugssysteme:
SELECT DISTINCT r.srid, r.coord_ref_sys_name, m.unit_of_meas_name,
                m.unit_of_meas_type
FROM SDO_UNITS_OF_MEASURE m, SDO_COORD_AXES a, SDO_COORD_REF_SYS r
WHERE m.uom_id = a.uom_id AND a.coord_sys_id = r.coord_sys_id AND
      r.srid IN (2056, 3416, 25832, 31467, 262148, 900913)
ORDER BY r.srid;

CREATE TABLE GeoGemeinden (
  code      VARCHAR(20),                 -- Gemeindeschlüssel
  name      VARCHAR(200)  NOT NULL,      -- Gemeindename
  status    VARCHAR(30),                 -- Status der Gemeinde
  wdcode    VARCHAR(15),                 -- Schlüssel für WikiData
  hektar    DECIMAL(10,2),               -- Fläche in Hektar
  zensus11  DECIMAL(7),                  -- Einwohnerzahl Zensus 2011
  einw19    DECIMAL(7),                  -- Einwohnerzahl Ende 2019
  geom      SDO_GEOMETRY,                -- Gemeindezentrum WGS84
  CONSTRAINT pk_ggem PRIMARY KEY (code)  -- Primärschlüssel
);

CREATE TABLE Ortsnetze (
  vorwahl   VARCHAR(6),                     -- Vorwahlnummer
  name      VARCHAR(80),                    -- Name des Ortsnetzes
  gebiet    SDO_GEOMETRY,                   -- Gebiet des Ortsnetzes
  CONSTRAINT pk_onetz PRIMARY KEY(vorwahl)  -- Primärschlüssel
);

INSERT INTO USER_SDO_GEOM_METADATA
VALUES ('GEOGEMEINDEN', 'GEOM', SDO_DIM_ARRAY(
                            SDO_DIM_ELEMENT('Länge', -180,180, 100),
                            SDO_DIM_ELEMENT('Breite', -90,90, 100)), 4326);
INSERT INTO USER_SDO_GEOM_METADATA
VALUES ('ORTSNETZE', 'GEBIET', SDO_DIM_ARRAY(
                            SDO_DIM_ELEMENT('Länge', -180,180, 100),
                            SDO_DIM_ELEMENT('Breite', -90,90, 100)), 4326);
COMMIT;

-- Hier GeoGemeindenSDOGeom.sql ausführen.

-- Hier OrtsnetzeSDOGeom.sql ausführen.

SELECT * FROM Ortsnetze WHERE vorwahl LIKE '04%'

SELECT DISTINCT (geom).SDO_SRID FROM GeoGemeinden;

SELECT SDO_GEOM.SDO_BUFFER(geom,5000,10,'arc_tolerance=50')
FROM GeoGemeinden
WHERE code LIKE 'DE05%';

-- Fläche und Umfang in km² bzw. km: 
SELECT name, SDO_GEOM.SDO_AREA(gebiet,1,'unit=sq_km') AS fläche_qkm,
             SDO_GEOM.SDO_LENGTH(gebiet,1,'unit=km') AS umfang_km
FROM Ortsnetze
WHERE name = 'Berlin' OR name = 'Hamburg';

-- Abstand in km: 
SELECT SDO_GEOM.SDO_DISTANCE (g1.geom, g2.geom, 100, 'unit=KM') 
FROM GeoGemeinden g1, GeoGemeinden g2
WHERE g1.name = 'Berlin' AND g2.name = 'Hamburg';

-- 4.2.1.8 Koordinatentransformationen

-- Transformation nach ETRS 89/UTM 32N: 
SELECT code, (geom).Get_WKT() AS wgs,
             SDO_CS.TRANSFORM(geom,25832).Get_WKT() AS etrs
FROM GeoGemeinden ORDER BY code;

-- Pufferzone mit 5000 m um die Gemeindezentren in Nordrhein-Westfalen: 
SELECT SDO_GEOM.SDO_BUFFER(SDO_CS.TRANSFORM(geom,25832), 5000, 10)
FROM GeoGemeinden WHERE code LIKE 'DE05%';

-- Abstand zwischen Berlin und Hamburg in km: 
SELECT SDO_GEOM.SDO_DISTANCE(SDO_CS.TRANSFORM(g1.geom,25832),
                             SDO_CS.TRANSFORM(g2.geom,25832),10)/1000 AS km
FROM GeoGemeinden g1, GeoGemeinden g2
WHERE g1.name = 'Berlin' AND g2.name = 'Hamburg';

-- 4.2.1.9 Geometrische Aggregatfunktionen

-- Aggregation der Gemeindezentren in Nordrhein-Westfalen: 
SELECT SDO_AGGR_UNION( SDOAGGRTYPE(geom,10) ) FROM GeoGemeinden
WHERE code LIKE 'DE05%';

-- Fläche des MUR und der konvexen Hülle der NRW-Gemeindezentren in km²: 
SELECT SDO_GEOM.SDO_AREA(SDO_AGGR_MBR(geom),5,'unit=SQ_KM'),
  SDO_GEOM.SDO_AREA(SDO_AGGR_CONVEXHULL(SDOAGGRTYPE(geom,5)),5,'unit=SQ_KM')
FROM GeoGemeinden
WHERE code LIKE 'DE05%';

-- Fläche der konvexen Hülle pro Landkreis: 
SELECT substr(code,1,7) AS landkreis,
  SDO_GEOM.SDO_AREA(SDO_AGGR_CONVEXHULL(SDOAGGRTYPE(geom,5)),5,'unit=SQ_KM')
FROM GeoGemeinden
WHERE code LIKE 'DE05%'
GROUP BY substr(code,1,7)
HAVING count(geom) > 1
ORDER BY substr(code,1,7);

-- 4.2.2.1 Deklaration und Konstruktoren

CREATE TABLE MMSpatialLand (
  id     INTEGER,                         -- Identifikationsnummer
  name   VARCHAR(20),                     -- Objektname
  geo    ST_GEOMETRY,                     -- Objektgeometrie
  CONSTRAINT pk_mmspland PRIMARY KEY(id)  -- Primärschlüssel
);

-- SFA-Konstruktor Point(x,y):
INSERT INTO MMSpatialLand (id,name,geo)
VALUES (1,'Flaggenmast', POINT(10,11));

-- SQL/MM Spatial Konstruktor ST_LineString(ARRAY OF ST_Point):
INSERT INTO MMSpatialLand (id,name,geo)
VALUES (2,'Steg', ST_LINESTRING(
  ST_POINT_ARRAY( ST_POINT(4,4), ST_POINT(5,7), ST_POINT(8,8) ) ) );
  
-- SQL/MM Spatial WKT-Methode für ST_MultiPoint:
INSERT INTO MMSpatialLand (id,name,geo)
VALUES (3,'Baumreihe', ST_MULTIPOINT.From_WKT(
  'MULTIPOINT ( (10 19) , (12 19) , (14 19) )') );
  
-- SQL/MM Spatial WKT-Methode für ST_CircularString:
INSERT INTO MMSpatialLand (id,name,geo)
VALUES (4,'Wall',ST_CIRCULARSTRING.From_WKT(
  'CIRCULARSTRING (1 15, 2.414 15.586, 3 17, 3.586 18.414, 5 19)') );
  
-- SQL/MM Spatial WKT-Methode für ST_CompoundCurve:
UPDATE MMSpatialLand SET geo = ST_COMPOUNDCURVE.From_WKT(
  'COMPOUNDCURVE ( CIRCULARSTRING (1 15, 2.414 15.586, 3 17, 3.586 18.414,
                                   5 19) , (5 19, 7 19) )')
WHERE name = 'Wall';

-- SQL/MM Spatial Konstruktor ST_Polygon(ST_LineString):
INSERT INTO MMSpatialLand (id,name,geo)
VALUES (5,'Haus', ST_POLYGON( ST_LINESTRING (
  ST_POINT_ARRAY(ST_POINT(9,9), ST_POINT(12,9), ST_POINT(12,10),
  ST_POINT(9,10), ST_POINT(9,9) ) ) ) );
  
-- Oracle-Konstruktor ST_GEOMETRY(SDO_GEOMETRY)
INSERT INTO MMSpatialLand (id,name,geo)
VALUES (6,'Land',
        ST_MULTIPOLYGON( (SELECT geo FROM GeoDbLand WHERE id = 6) ) );
COMMIT;

-- Einfügen von Metadaten für ST_GEOMETRY:
INSERT INTO USER_SDO_GEOM_METADATA
VALUES ('MMSPATIALLAND', 'GEO', SDO_DIM_ARRAY(
                        SDO_DIM_ELEMENT('X', 0,20, 0.005),
                        SDO_DIM_ELEMENT('Y', 0,20, 0.005) ), NULL);
COMMIT;

-- 4.2.2.2 Grundfunktionalität

SELECT name, ST_GEOMETRY.ST_AsText(geo) FROM MMSpatialLand;

SELECT name, (geo).ST_Dimension() FROM MMSpatialLand;

SELECT DISTINCT (geo).ST_CoordDim() FROM MMSpatialLand;

SELECT name, (geo).ST_GeometryType() FROM MMSpatialLand;

SELECT name, (geo).ST_GetTolerance() FROM MMSpatialLand WHERE id = 1;

-- Zugriff auf Punktkoordinaten mit ST_X und ST_Y: 
SELECT TREAT( geo AS ST_POINT ).ST_X() AS x, 
       TREAT( geo AS ST_POINT ).ST_Y() AS y
FROM MMSpatialLand
WHERE geo IS OF (ST_POINT);

-- Zugriff auf Linienendpunkte mit ST_StartPoint, ST_PointN und ST_EndPoint: 
SELECT name, TREAT(geo AS ST_CURVE).ST_StartPoint() AS s,
       TREAT(geo AS ST_CURVE).ST_PointN( ROUND(
         TREAT(geo AS ST_CURVE).ST_NumPoints() / 2) ) AS m,
       TREAT(geo AS ST_CURVE).ST_EndPoint() AS e
FROM MMSpatialLand
WHERE geo IS OF (ST_CURVE);

-- Zugriff auf Linienpunkte mit ST_Points: 
SELECT name, TREAT(geo AS ST_CURVE).ST_Points() AS punktfeld
FROM MMSpatialLand
WHERE geo IS OF (ST_CURVE);

-- Zugriff auf Linienpunkte mit Umwandlung in Relation: 
SELECT name, t.*
FROM MMSpatialLand, TABLE(TREAT(geo AS ST_LINESTRING).ST_Points()) t 
WHERE name = 'Steg';

-- Zugriff auf Außenring eines Polygons mit ST_ExteriorRing: 
SELECT TREAT(geo AS ST_POLYGON).ST_ExteriorRing() AS aussenring
FROM MMSpatialLand
WHERE geo IS OF (ST_POLYGON);

-- Zugriff auf Teilgeometrien von Geometriesammlungen mit ST_Geometries:
SELECT name, TREAT(geo AS ST_MULTIPOLYGON).ST_Geometries() AS gfeld
FROM MMSpatialLand
WHERE geo IS OF (ST_MULTIPOLYGON);

-- Zugriff auf Teilgeometrien mit Umwandlung in Relation:
SELECT name, t.*
FROM MMSpatialLand,
     TABLE(TREAT(geo AS ST_GEOMCOLLECTION).ST_Geometries()) t 
WHERE geo IS OF (ST_GEOMCOLLECTION);

SELECT name, (geo).ST_Boundary() FROM MMSpatialLand g;

-- die bestehenden Geometrien überprüfen: 
SELECT (geo).ST_IsValid() FROM MMSpatialLand;
-- eine neue Geometrie überprüfen: 
SELECT  ST_POLYGON.From_WKT('POLYGON((9 9,12 10,12 9,9 10,9 9))').ST_IsValid()
FROM Dual;

-- 4.2.2.3 Geometrische Funktionen

SELECT g1.name, g2.name, g1.geo.ST_Distance(g2.geo)
FROM MMSpatialLand g1 INNER JOIN MMSpatialLand g2
ON g1.id < g2.id
ORDER BY g1.id, g2.id;

SELECT name, TREAT(geo AS ST_CURVE).ST_Length() AS länge
FROM MMSpatialLand WHERE geo IS OF (ST_CURVE);

SELECT name, TREAT(geo AS ST_SURFACE).ST_Area() AS fläche
FROM MMSpatialLand
WHERE geo IS OF (ST_SURFACE)
UNION ALL
SELECT name, TREAT(geo AS MDSYS.ST_MULTISURFACE).ST_Area()
FROM MMSpatialLand
WHERE geo IS OF (MDSYS.ST_MULTISURFACE);

SELECT name, (geo).ST_Centroid() AS centroid
FROM MMSpatialLand
WHERE (geo).ST_Centroid().ST_IsEmpty() = 0;

-- minimal umgebendes Rechteck (MUR):
SELECT geo, (geo).ST_Envelope() FROM MMSpatialLand WHERE name = 'Wall';
-- konvexe Hülle: 
SELECT geo, (geo).ST_ConvexHull() FROM MMSpatialLand WHERE name = 'Steg';

SELECT geo, (geo).ST_Buffer(2)
FROM MMSpatialLand WHERE name <> 'Land';

SELECT g1.geo.ST_Difference(g2.geo)
FROM MMSpatialLand g1, MMSpatialLand g2
WHERE g1.name = 'Land' AND g2.name = 'Haus';

-- 4.2.2.4 Prüfung topologischer Prädikate

SELECT g1.geo.ST_Intersects(g2.geo) AS INTERSECTS,
       g1.geo.ST_Touches (g2.geo) AS TOUCHES,
       g1.geo.ST_Within(g2.geo) + g1.geo.ST_Equals(g2.geo) AS WITHIN_EQ,
       g1.geo.ST_Relate(g2.geo, 'T*F**F***') AS RELATE
FROM MMSpatialLand g1, MMSpatialLand g2
WHERE g1.name = 'Haus' AND g2.name = 'Land';

-- 4.2.2.5 Koordinatenbezugssysteme

CREATE TABLE MMGemeinden (
  code      VARCHAR(20),                 -- Gemeindeschlüssel
  name      VARCHAR(200)  NOT NULL,      -- Gemeindename
  status    VARCHAR(30),                 -- Status der Gemeinde
  wdcode    VARCHAR(15),                 -- Schlüssel für WikiData
  hektar    DECIMAL(10,2),               -- Fläche in Hektar
  zensus11  DECIMAL(7),                  -- Einwohnerzahl Zensus 2011
  einw19    DECIMAL(7),                  -- Einwohnerzahl Ende 2019
  geom      ST_POINT,                    -- Gemeindezentrum WGS84
  CONSTRAINT pk_mmgem PRIMARY KEY (code) -- Primärschlüssel
);

INSERT INTO USER_SDO_GEOM_METADATA
VALUES ('MMGEMEINDEN', 'GEOM', SDO_DIM_ARRAY(
                            SDO_DIM_ELEMENT('Länge', -180,180, 100),
                            SDO_DIM_ELEMENT('Breite', -90,90, 100)), 4326);
COMMIT;

-- Hier MMGemeinden.sql ausführen.

SELECT DISTINCT (geom).ST_SRID() FROM MMGemeinden;


-- Pufferzone mit 5 km Abstand um die Gemeindezentren in Nordrhein-Westfalen:
SELECT (geom).ST_Buffer(5000) FROM MMGemeinden
WHERE code LIKE 'DE05%';

-- Abstand zwischen Berlin und Hamburg: 
SELECT g1.geom.ST_Distance(g2.geom)/1000 AS dist_km
FROM MMGemeinden g1, MMGemeinden g2
WHERE g1.name = 'Berlin' AND g2.name = 'Hamburg';

-- Transformation nach ETRS 89/UTM 32N: 
SELECT code, (geom).Get_WKT() AS wgs,
       ST_Geometry(SDO_CS.TRANSFORM((geom).geom,25832)).Get_WKT() AS etrs
FROM MMGemeinden ORDER BY code;

-- 4.2.3.1 Repräsentation

CREATE TABLE BWSAbschnitte (
  bws_id      VARCHAR(5),                      -- ID der Bundeswasserstraße
  stat_von    NUMERIC(10,6),                   -- Stationswert am Anfang
  stat_nach   NUMERIC(10,6),                   -- Stationswert am Ende
  route_id    VARCHAR(20),                     -- ID des Abschnitts
  stat_r      VARCHAR(10),                     -- Stationierungsrichtung
  geom        SDO_GEOMETRY,                    -- Geometrie
  CONSTRAINT bwsabs_pk PRIMARY KEY (route_id)  -- Primärschlüssel
);

SELECT route_id, stat_von, stat_nach, stat_r, geom AS wkt
FROM BWSAbschnitte WHERE bws_id = '0409' ORDER BY route_id;

SELECT SDO_AGGR_MBR(geom) FROM BWSAbschnitte;

SELECT min(stat_von), max(stat_von), min(stat_nach), max(stat_nach)
FROM BWSAbschnitte;

INSERT INTO USER_SDO_GEOM_METADATA
VALUES ('BWSABSCHNITTE', 'GEOM', SDO_DIM_ARRAY(
                       SDO_DIM_ELEMENT('X', 297000,5270000, 0.01),
                       SDO_DIM_ELEMENT('Y', 895000,6060000, 0.01) ), 25832);
COMMIT;

BEGIN
  IF SDO_LRS.CONVERT_TO_LRS_LAYER('BWSABSCHNITTE','GEOM',
                                  0, 2500, 0.01) = 'TRUE' THEN
     RETURN;  -- hier ggf. weitere Arbeitsschritte
  END IF;
END;
/

SELECT (geom).SDO_GTYPE AS gtype, (geom).SDO_ORDINATES AS ordinates
FROM BWSAbschnitte WHERE bws_id = '0409' ORDER BY route_id;

SELECT diminfo FROM USER_SDO_GEOM_METADATA WHERE table_name='BWSABSCHNITTE';

DECLARE
  CURSOR c IS SELECT * FROM BWSAbschnitte FOR UPDATE; -- DB-Cursor 
  abs c%ROWTYPE;                                      -- aktueller Abschnitt
BEGIN
  FOR abs IN c LOOP   -- alle Abschnitte durchlaufen
    -- Maßzahlen in der Geometrie setzen: 
    IF abs.stat_r = 'gleich' THEN
      SDO_LRS.REDEFINE_GEOM_SEGMENT(abs.geom,abs.stat_von,abs.stat_nach);
    ELSE
      SDO_LRS.REDEFINE_GEOM_SEGMENT(abs.geom,abs.stat_nach,abs.stat_von);
    END IF;
    -- geänderte Geometrie speichern: 
    UPDATE BWSAbschnitte SET geom = abs.geom WHERE route_id = abs.route_id;
  END LOOP;
  COMMIT;
END;
/

SELECT stat_von AS von, stat_nach AS nach, stat_r AS r,
       (geom).SDO_ORDINATES AS ordinates
FROM BWSAbschnitte
WHERE bws_id = '0409' ORDER BY route_id;

-- 4.2.3.2 Anfragen

SELECT DISTINCT (geom).Get_LRS_Dim() FROM BWSAbschnitte;

SELECT route_id, SDO_LRS.GEOM_SEGMENT_START_MEASURE(geom) AS m1,
       SDO_LRS.GET_MEASURE(SDO_LRS.GEOM_SEGMENT_END_PT(geom)) AS m2
FROM BWSAbschnitte
WHERE bws_id = '0409' ORDER BY route_id;

SELECT route_id, t.id, t.x, t.y, t.z AS m
FROM (SELECT * FROM BWSAbschnitte WHERE bws_id = '0409') b,
     TABLE(SDO_UTIL.GETVERTICES(b.geom)) t
ORDER BY route_id, t.id;

SELECT route_id, abs(stat_nach - stat_von), SDO_LRS.MEASURE_RANGE(geom)
FROM BWSAbschnitte
WHERE bws_id = '0409' ORDER BY route_id;

-- Berechnung eines Punktes ohne und mit Versatzentfernung: 
SELECT route_id, SDO_LRS.LOCATE_PT(geom,65), SDO_LRS.LOCATE_PT(geom,65,500)
FROM BWSAbschnitte
WHERE bws_id = '0101' AND SDO_LRS.VALID_MEASURE(geom,65) = 'TRUE';

-- Berechnung eines Streckenzugs: 
SELECT route_id, SDO_LRS.DYNAMIC_SEGMENT(geom,66,78)
FROM BWSAbschnitte
WHERE bws_id = '0101' AND SDO_LRS.VALID_MEASURE(geom,66) = 'TRUE' AND
                          SDO_LRS.VALID_MEASURE(geom,78) = 'TRUE';

SELECT SDO_LRS.FIND_MEASURE (geom,
                             SDO_GEOMETRY('POINT(534202 5847101)',25832))
FROM BWSAbschnitte
WHERE route_id = '0101_0052.248000';

SELECT SDO_LRS.FIND_OFFSET(geom,
       SDO_GEOMETRY('POINT(534669.166923905 5846921.29050257)',25832))
FROM BWSAbschnitte
WHERE route_id = '0101_0052.248000';

SELECT SDO_LRS.PROJECT_PT(geom, SDO_LRS.LOCATE_PT(geom,65,500))
FROM BWSAbschnitte
WHERE bws_id = '0101' AND SDO_LRS.VALID_MEASURE(geom,65) = 'TRUE';




