-- ---------------------
-- KAPITEL 5 (Oracle 21)
-- ---------------------

-- 5.5.2.1 Erzeugung eines räumlichen Index

-- neuen Index anlegen:
CREATE INDEX GeoDbLand_geo_ix ON GeoDbLand(geo)
INDEXTYPE IS MDSYS.SPATIAL_INDEX_V2;

-- auch für ST_GEOMETRY möglich:
CREATE INDEX MMSpatialLand_geo_ix ON MMSpatialLand(geo)
INDEXTYPE IS MDSYS.SPATIAL_INDEX_V2;

CREATE INDEX MMGemeinden_geom_ix ON MMGemeinden(geom)
INDEXTYPE IS MDSYS.SPATIAL_INDEX_V2;

-- Räumlichen Index für GeoGemeinden und Ortsnetze anlegen:
CREATE INDEX GeoGemeinden_geom_ix ON GeoGemeinden(geom)
INDEXTYPE IS MDSYS.SPATIAL_INDEX_V2 PARAMETERS('LAYER_GTYPE=POINT');

CREATE INDEX Ortsnetze_gebiet_ix ON Ortsnetze(gebiet)
INDEXTYPE IS MDSYS.SPATIAL_INDEX_V2 PARAMETERS('LAYER_GTYPE=MULTIPOLYGON');

-- Versuch, ein Polygon in GeoGemeinden zu speichern:
UPDATE GeoGemeinden SET geom =
  SDO_GEOMETRY (2003, 4326,NULL, SDO_ELEM_INFO_ARRAY(1,1003,1),
            SDO_ORDINATE_ARRAY(8.214289,53.087776, 8.305773, 53.148083,
                               8.173382,53.184449, 8.214289,53.087776) )
WHERE code = 'DE03403000';

SELECT INDEX_NAME, TABLE_NAME, COLUMN_NAME, SDO_INDEX_TYPE, SDO_INDEX_TABLE
FROM USER_SDO_INDEX_INFO ORDER BY INDEX_NAME;

-- 5.5.2.2 Filterung über SDO_FILTER

-- Anfrage ausführen:
SELECT name, geo, SDO_GEOMETRY('POLYGON((5 5, 5 15, 15 15, 15 5, 5 5))')
FROM GeoDbLand
WHERE SDO_FILTER (geo, 
          SDO_GEOMETRY('POLYGON((5 5, 5 15, 15 15, 15 5, 5 5))')) = 'TRUE';

-- Auch auf ST_GEOMETRY anwendbar:
SELECT name
FROM MMSpatialLand
WHERE SDO_FILTER (geo, ST_POLYGON.FROM_WKT(
                  'POLYGON((5 5, 5 15, 15 15, 15 5, 5 5))')) = 'TRUE';

EXPLAIN PLAN FOR
  SELECT name FROM GeoDbLand WHERE SDO_FILTER (geo, 
     SDO_GEOMETRY('POLYGON((5 5, 5 15, 15 15, 15 5, 5 5))')) = 'TRUE';

SELECT n1.name
FROM Ortsnetze n1 INNER JOIN Ortsnetze n2
ON SDO_FILTER (n1.gebiet, n2.gebiet) = 'TRUE'
WHERE n2.vorwahl = '04431';

-- 5.5.2.3 Verfeinerungsschritt über SDO_RELATE

-- Rechteckanfrage:
SELECT name
FROM GeoDbLand
WHERE SDO_RELATE(geo,SDO_GEOMETRY('POLYGON((5 5, 5 15, 15 15, 15 5, 5 5))'),
                 'querytype=WINDOW mask=ANYINTERACT') = 'TRUE';

-- Rechteckanfrage angewendet auf ST_GEOMETRY:
SELECT name
FROM MMSpatialLand
WHERE SDO_RELATE(geo,ST_POLYGON.FROM_WKT(
                                  'POLYGON((5 5, 5 15, 15 15, 15 5, 5 5))'),
                 'querytype=WINDOW mask=ANYINTERACT') = 'TRUE';

-- Rechteckanfrage mittels SDO_ANYINTERACT:
SELECT name
FROM GeoDbLand
WHERE SDO_ANYINTERACT (geo,
          SDO_GEOMETRY('POLYGON((5 5, 5 15, 15 15, 15 5, 5 5))')) = 'TRUE';

SELECT name
FROM GeoDbLand
WHERE SDO_INSIDE(geo,
          SDO_GEOMETRY('POLYGON((5 5, 5 15, 15 15, 15 5, 5 5))')) = 'TRUE';

-- 5.5.2.4 SDO_WITHIN_DISTANCE

SELECT netz.name,
       SDO_GEOM.SDO_DISTANCE(netz.gebiet,gem.geom,1e-5,'unit=KM')
FROM Ortsnetze netz INNER JOIN GeoGemeinden gem
ON SDO_WITHIN_DISTANCE (netz.gebiet, gem.geom,
                        'distance=5 querytype=FILTER unit=KM') = 'TRUE'
WHERE gem.code = 'DE03451002';

SELECT g1.name, g1.geo.ST_DISTANCE(g2.geo)
FROM MMSpatialLand g1 INNER JOIN MMSpatialLand g2
ON SDO_WITHIN_DISTANCE (g1.geo, g2.geo,'distance=5') = 'TRUE'
WHERE g2.id = 1;
