-- ---------------------
-- KAPITEL 6 (Oracle 21)
-- ---------------------

-- 6.3.6.1 Fixed Indexing

DROP INDEX GeoDbLand_geo_ix;    -- ggf. vorhandenen Index löschen

CREATE INDEX GeoDbLand_geo_ix ON GeoDbLand(geo)
INDEXTYPE IS MDSYS.SPATIAL_INDEX PARAMETERS('SDO_LEVEL=6');

ALTER INDEX GeoDbLand_geo_ix REBUILD PARAMETERS('SDO_LEVEL=4');

-- Anfrage bei SDO_LEVEL=4 ausführen:
SELECT name FROM GeoDbLand
WHERE SDO_FILTER (geo, 
      SDO_GEOMETRY('POLYGON((3.5 5, 3.5 16, 15 16, 15 5, 3.5 5))')) = 'TRUE';

-- 6.3.6.2 Hybrid Indexing

ALTER INDEX GeoDbLand_geo_ix
REBUILD PARAMETERS('SDO_LEVEL=6 SDO_NUMTILES=10');

SELECT INDEX_NAME, TABLE_NAME, COLUMN_NAME, SDO_INDEX_TYPE, SDO_INDEX_TABLE
FROM USER_SDO_INDEX_INFO
WHERE SDO_INDEX_TYPE = 'QTREE';

SELECT SDO_INDEX_NAME,SDO_INDEX_TABLE,SDO_LEVEL,SDO_NUMTILES,SDO_MAXLEVEL
FROM USER_SDO_INDEX_METADATA
WHERE SDO_INDEX_NAME = 'GEODBLAND_GEO_IX';

-- Die nachfolgend verwendete Tabelle MDQT_774C$ durch die angezeigte SDO_INDEX_TABLE ersetzen!

-- Fix Indexing mit grober Zerlegung:
ALTER INDEX GeoDbLand_geo_ix REBUILD PARAMETERS('SDO_LEVEL=4');

-- Bestimmung der Anzahl der z-Werte pro Geometrie:
SELECT l.name, count(i.sdo_code)
FROM MDQT_774C$ i INNER JOIN GeoDbLand l
ON i.sdo_rowid = l.rowid
GROUP BY l.name;

-- Fix Indexing mit feinerer Zerlegung:
ALTER INDEX GeoDbLand_geo_ix REBUILD PARAMETERS('SDO_LEVEL=6');

-- Bestimmung der Anzahl der z-Werte pro Geometrie:
SELECT l.name, count(i.sdo_code)
FROM MDQT_774C$ i INNER JOIN GeoDbLand l
ON i.sdo_rowid ON l.rowid
GROUP BY l.name;

-- Hybrid Indexing:
ALTER INDEX GeoDbLand_geo_ix
REBUILD PARAMETERS('SDO_LEVEL=4 SDO_NUMTILES=10');

-- Bestimmung der Anzahl der z-Werte pro Geometrie:
SELECT l.name, count(i.sdo_code)
FROM MDQT_774C$ i INNER JOIN GeoDbLand l
ON i.sdo_rowid = l.rowid
GROUP BY l.name;

SELECT l.name, i.sdo_groupcode, count(i.sdo_code)
FROM MDQT_774C$ i INNER JOIN GeoDbLand l
ON i.sdo_rowid = l.rowid 
WHERE l.name = 'Haus'
GROUP BY l.name, i.sdo_groupcode;

-- Fix Indexing mit feiner Zerlegung:
ALTER INDEX GeoDbLand_geo_ix REBUILD PARAMETERS('SDO_LEVEL=6');

-- Betrachtung der Statuswerte der Zellen beim Haus:
SELECT l.name, i.sdo_status, count(i.sdo_code)
FROM MDQT_774C$ i INNER JOIN GeoDbLand l
ON i.sdo_rowid = l.rowid 
WHERE l.name = 'Haus'
GROUP BY l.name, i.sdo_status;


-- 6.5.5.1 Metadaten

SELECT SDO_INDEX_NAME AS index_name, SDO_RTREE_HEIGHT AS height,
       SDO_RTREE_NUM_NODES AS num, SDO_RTREE_FANOUT AS fanout,
       SDO_RTREE_ROOT AS root, SDO_LAYER_GTYPE AS gtype
FROM USER_SDO_INDEX_METADATA
WHERE SDO_INDEX_TYPE = 'RTREE'
ORDER BY SDO_INDEX_NAME;

-- Index ändern:
ALTER INDEX Ortsnetze_gebiet_ix
REBUILD PARAMETERS('SDO_NON_LEAF_TBL=TRUE');

-- Tabellennamen anzeigen:
SELECT t.table_name
FROM USER_TABLES t INNER JOIN USER_SDO_INDEX_INFO i
ON substr(t.table_name,5,5) = substr(i.sdo_index_table,5,5)
WHERE i.index_name = 'ORTSNETZE_GEBIET_IX';

-- 6.5.5.2 Funktionsbasierter R-Baum

CREATE OR REPLACE FUNCTION schwerpunkt (geo SDO_GEOMETRY)
                                        RETURN SDO_GEOMETRY DETERMINISTIC
IS
BEGIN
  RETURN SDO_GEOM.SDO_CENTROID(geo,0.001);
END;
/

INSERT INTO USER_SDO_GEOM_METADATA
VALUES ('ORTSNETZE','<Schemaname>.SCHWERPUNKT(GEBIET)', SDO_DIM_ARRAY(
  SDO_DIM_ELEMENT('X',-180,180,1),
  SDO_DIM_ELEMENT('Y',-90,90,1)
) , 4326);
COMMIT;

CREATE INDEX Ortsnetze_gebiet_spkt_ix
ON Ortsnetze (schwerpunkt(gebiet))
INDEXTYPE IS MDSYS.SPATIAL_INDEX_V2;

SELECT * FROM Ortsnetze
WHERE SDO_FILTER (schwerpunkt(gebiet),
        SDO_GEOMETRY('POLYGON((8 53,8 54,9 54,9 53,8 53))',4326)) = 'TRUE';

EXPLAIN PLAN FOR
  SELECT * FROM Ortsnetze
  WHERE SDO_FILTER (schwerpunkt(gebiet),
        SDO_GEOMETRY('POLYGON((8 53,8 54,9 54,9 53,8 53))',4326)) = 'TRUE';

-- 6.5.5.3 Index über GeoJSON

CREATE INDEX GeoDbLandJson_geo_idx
ON GeoDbLandJson (json_value(doc, '$.geo' RETURNING SDO_GEOMETRY))
INDEXTYPE IS MDSYS.SPATIAL_INDEX_V2;

SELECT TABLE_NAME, COLUMN_NAME, SRID 
FROM USER_SDO_GEOM_METADATA
WHERE TABLE_NAME = 'GEODBLANDJSON';

SELECT id, j.doc.name.string(), j.doc.geo.type.string()
FROM GeoDbLandJson j
WHERE SDO_RELATE (json_value(doc, '$.geo' RETURNING SDO_GEOMETRY), 
           SDO_GEOMETRY('POLYGON((5 5, 5 15, 15 15, 15 5, 5 5))',4326),
           'querytype=WINDOW mask=ANYINTERACT') = 'TRUE'
ORDER BY j.doc.id.number();

